@@ -131,8 +131,8 @@ namespace Google.Protobuf
             // Without setting any values, there's nothing to write.
             byte[] bytes = message.ToByteArray();
             Assert.AreEqual(0, bytes.Length);
-
-            MessageParsingHelpers.AssertRoundtrip(TestAllTypes.Parser, message);
+            TestAllTypes parsed = TestAllTypes.Parser.ParseFrom(bytes);
+            Assert.AreEqual(message, parsed);
         }
 
         [Test]
@@ -165,8 +165,8 @@ namespace Google.Protobuf
             };
 
             byte[] bytes = message.ToByteArray();
-
-            MessageParsingHelpers.AssertRoundtrip(TestAllTypes.Parser, message);
+            TestAllTypes parsed = TestAllTypes.Parser.ParseFrom(bytes);
+            Assert.AreEqual(message, parsed);
         }
 
         [Test]
@@ -199,8 +199,8 @@ namespace Google.Protobuf
             };
 
             byte[] bytes = message.ToByteArray();
-
-            MessageParsingHelpers.AssertRoundtrip(TestAllTypes.Parser, message);
+            TestAllTypes parsed = TestAllTypes.Parser.ParseFrom(bytes);
+            Assert.AreEqual(message, parsed);
         }
 
         // Note that not every map within map_unittest_proto3 is used. They all go through very
@@ -231,8 +231,8 @@ namespace Google.Protobuf
             };
 
             byte[] bytes = message.ToByteArray();
-
-            MessageParsingHelpers.AssertRoundtrip(TestMap.Parser, message);
+            TestMap parsed = TestMap.Parser.ParseFrom(bytes);
+            Assert.AreEqual(message, parsed);
         }
 
         [Test]
@@ -246,14 +246,9 @@ namespace Google.Protobuf
             byte[] bytes = message.ToByteArray();
             Assert.AreEqual(2, bytes.Length); // Tag for field entry (1 byte), length of entry (0; 1 byte)
 
-            MessageParsingHelpers.AssertReadingMessage(
-                TestMap.Parser,
-                bytes,
-                parsed=>
-                {
-                    Assert.AreEqual(1, parsed.MapInt32Bytes.Count);
-                    Assert.AreEqual(ByteString.Empty, parsed.MapInt32Bytes[0]);
-                });
+            var parsed = TestMap.Parser.ParseFrom(bytes);
+            Assert.AreEqual(1, parsed.MapInt32Bytes.Count);
+            Assert.AreEqual(ByteString.Empty, parsed.MapInt32Bytes[0]);
         }
 
         [Test]
@@ -270,13 +265,8 @@ namespace Google.Protobuf
             output.WriteMessage(nestedMessage);
             output.Flush();
 
-            MessageParsingHelpers.AssertReadingMessage(
-                TestMap.Parser,
-                memoryStream.ToArray(),
-                parsed =>
-                {
-                    Assert.AreEqual(nestedMessage, parsed.MapInt32ForeignMessage[0]);
-                });
+            var parsed = TestMap.Parser.ParseFrom(memoryStream.ToArray());
+            Assert.AreEqual(nestedMessage, parsed.MapInt32ForeignMessage[0]);
         }
 
         [Test]
@@ -292,13 +282,8 @@ namespace Google.Protobuf
             output.WriteInt32(key);
             output.Flush();
 
-            MessageParsingHelpers.AssertReadingMessage(
-                TestMap.Parser,
-                memoryStream.ToArray(),
-                parsed =>
-                {
-                    Assert.AreEqual(0.0, parsed.MapInt32Double[key]);
-                });
+            var parsed = TestMap.Parser.ParseFrom(memoryStream.ToArray());
+            Assert.AreEqual(0.0, parsed.MapInt32Double[key]);
         }
 
         [Test]
@@ -314,13 +299,8 @@ namespace Google.Protobuf
             output.WriteInt32(key);
             output.Flush();
 
-            MessageParsingHelpers.AssertReadingMessage(
-                TestMap.Parser,
-                memoryStream.ToArray(),
-                parsed =>
-                {
-                    Assert.AreEqual(new ForeignMessage(), parsed.MapInt32ForeignMessage[key]);
-                });
+            var parsed = TestMap.Parser.ParseFrom(memoryStream.ToArray());
+            Assert.AreEqual(new ForeignMessage(), parsed.MapInt32ForeignMessage[key]);
         }
 
         [Test]
@@ -347,13 +327,8 @@ namespace Google.Protobuf
             output.WriteInt32(extra);
             output.Flush();
 
-            MessageParsingHelpers.AssertReadingMessage(
-                TestMap.Parser,
-                memoryStream.ToArray(),
-                parsed =>
-                {
-                    Assert.AreEqual(value, parsed.MapInt32Int32[key]);
-                });
+            var parsed = TestMap.Parser.ParseFrom(memoryStream.ToArray());
+            Assert.AreEqual(value, parsed.MapInt32Int32[key]);
         }
 
         [Test]
@@ -376,13 +351,8 @@ namespace Google.Protobuf
             output.WriteInt32(key);
             output.Flush();
 
-            MessageParsingHelpers.AssertReadingMessage(
-                TestMap.Parser,
-                memoryStream.ToArray(),
-                parsed =>
-                {
-                    Assert.AreEqual(value, parsed.MapInt32Int32[key]);
-                });
+            var parsed = TestMap.Parser.ParseFrom(memoryStream.ToArray());
+            Assert.AreEqual(value, parsed.MapInt32Int32[key]);
         }
 
         [Test]
@@ -427,19 +397,13 @@ namespace Google.Protobuf
             output.WriteInt32(value3);
 
             output.Flush();
-
-            MessageParsingHelpers.AssertReadingMessage(
-                TestMap.Parser,
-                memoryStream.ToArray(),
-                parsed =>
-                {
-                    var expected = new TestMap
-                    {
-                        MapInt32Int32 = { { key1, value1 }, { key3, value3 } },
-                        MapStringString = { { key2, value2 } }
-                    };
-                    Assert.AreEqual(expected, parsed);
-                });
+            var parsed = TestMap.Parser.ParseFrom(memoryStream.ToArray());
+            var expected = new TestMap
+            {
+                MapInt32Int32 = { { key1, value1 }, { key3, value3 } },
+                MapStringString = { { key2, value2 } }
+            };
+            Assert.AreEqual(expected, parsed);
         }
 
         [Test]
@@ -469,13 +433,8 @@ namespace Google.Protobuf
             output.WriteInt32(value2);
             output.Flush();
 
-            MessageParsingHelpers.AssertReadingMessage(
-                TestMap.Parser,
-                memoryStream.ToArray(),
-                parsed =>
-                {
-                    Assert.AreEqual(value2, parsed.MapInt32Int32[key]);
-                });
+            var parsed = TestMap.Parser.ParseFrom(memoryStream.ToArray());
+            Assert.AreEqual(value2, parsed.MapInt32Int32[key]);
         }
 
         [Test]
@@ -660,10 +619,9 @@ namespace Google.Protobuf
             var bytes = message.ToByteArray();
             Assert.AreEqual(3, bytes.Length); // 2 bytes for the tag + 1 for the value - no string!
 
-            MessageParsingHelpers.AssertRoundtrip(TestAllTypes.Parser, message, parsedMessage =>
-            {
-                Assert.AreEqual(TestAllTypes.OneofFieldOneofCase.OneofUint32, parsedMessage.OneofFieldCase);
-            });
+            var message2 = TestAllTypes.Parser.ParseFrom(bytes);
+            Assert.AreEqual(message, message2);
+            Assert.AreEqual(TestAllTypes.OneofFieldOneofCase.OneofUint32, message2.OneofFieldCase);
         }
 
         [Test]
@@ -675,10 +633,9 @@ namespace Google.Protobuf
             var bytes = message.ToByteArray();
             Assert.AreEqual(3, bytes.Length); // 2 bytes for the tag + 1 for the value - it's still serialized
 
-            MessageParsingHelpers.AssertRoundtrip(TestAllTypes.Parser, message, parsedMessage =>
-            {
-                Assert.AreEqual(TestAllTypes.OneofFieldOneofCase.OneofUint32, parsedMessage.OneofFieldCase);
-            });
+            var message2 = TestAllTypes.Parser.ParseFrom(bytes);
+            Assert.AreEqual(message, message2);
+            Assert.AreEqual(TestAllTypes.OneofFieldOneofCase.OneofUint32, message2.OneofFieldCase);
         }
 
         [Test]
@@ -694,14 +651,10 @@ namespace Google.Protobuf
             message.WriteTo(output);
             output.Flush();
 
-            MessageParsingHelpers.AssertReadingMessage(
-                TestAllTypes.Parser,
-                stream.ToArray(),
-                parsed =>
-                {
-                    // TODO(jieluo): Add test back when DiscardUnknownFields API is supported.
-                    // Assert.AreEqual(message, parsed);
-                });
+            stream.Position = 0;
+            var parsed = TestAllTypes.Parser.ParseFrom(stream);
+            // TODO(jieluo): Add test back when DiscardUnknownFields API is supported.
+            // Assert.AreEqual(message, parsed);
         }
 
         [Test]
@@ -710,15 +663,8 @@ namespace Google.Protobuf
             // Simple way of ensuring we can skip all kinds of fields.
             var data = SampleMessages.CreateFullTestAllTypes().ToByteArray();
             var empty = Empty.Parser.ParseFrom(data);
-
-            MessageParsingHelpers.AssertReadingMessage(
-                Empty.Parser,
-                data,
-                parsed =>
-                {
-                    // TODO(jieluo): Add test back when DiscardUnknownFields API is supported.
-                    // Assert.AreNotEqual(new Empty(), empty);
-                });
+            // TODO(jieluo): Add test back when DiscardUnknownFields API is supported.
+            // Assert.AreNotEqual(new Empty(), empty);
         }
 
         // This was originally seen as a conformance test failure.
@@ -728,7 +674,7 @@ namespace Google.Protobuf
             // 130, 3 is the message tag
             // 1 is the data length - but there's no data.
             var data = new byte[] { 130, 3, 1 };
-            MessageParsingHelpers.AssertReadingMessageThrows<TestAllTypes, InvalidProtocolBufferException>(TestAllTypes.Parser, data);
+            Assert.Throws<InvalidProtocolBufferException>(() => TestAllTypes.Parser.ParseFrom(data));
         }
 
         /// <summary>
@@ -749,7 +695,7 @@ namespace Google.Protobuf
             output.Flush();
 
             stream.Position = 0;
-            MessageParsingHelpers.AssertReadingMessageThrows<TestAllTypes, InvalidProtocolBufferException>(TestAllTypes.Parser, stream.ToArray());
+            Assert.Throws<InvalidProtocolBufferException>(() => TestAllTypes.Parser.ParseFrom(stream));
         }
 
         [Test]
