@@ -31,11 +31,9 @@
 #endregion
 
 using Google.Protobuf.Reflection;
-using System.Buffers;
 using System.Collections;
 using System.IO;
 using System.Linq;
-using System.Security;
 
 namespace Google.Protobuf
 {
@@ -250,16 +248,6 @@ namespace Google.Protobuf
             codedInput.CheckReadEndOfStreamTag();
         }
 
-        [SecuritySafeCritical]
-        internal static void MergeFrom(this IMessage message, ReadOnlySequence<byte> data, bool discardUnknownFields, ExtensionRegistry registry)
-        {
-            ParseContext.Initialize(data, out ParseContext ctx);
-            ctx.DiscardUnknownFields = discardUnknownFields;
-            ctx.ExtensionRegistry = registry;
-            ParsingPrimitivesMessages.ReadRawMessage(ref ctx, message);
-            ParsingPrimitivesMessages.CheckReadEndOfStreamTag(ref ctx.state);
-        }
-
         internal static void MergeDelimitedFrom(this IMessage message, Stream input, bool discardUnknownFields, ExtensionRegistry registry)
         {
             ProtoPreconditions.CheckNotNull(message, "message");
