@@ -45,7 +45,7 @@ import java.util.RandomAccess;
 final class LongArrayList extends AbstractProtobufList<Long>
     implements LongList, RandomAccess, PrimitiveNonBoxingCollection {
 
-  private static final LongArrayList EMPTY_LIST = new LongArrayList();
+  private static final LongArrayList EMPTY_LIST = new LongArrayList(new long[0], 0);
   static {
     EMPTY_LIST.makeImmutable();
   }
@@ -54,9 +54,7 @@ final class LongArrayList extends AbstractProtobufList<Long>
     return EMPTY_LIST;
   }
 
-  /**
-   * The backing store for the list.
-   */
+  /** The backing store for the list. */
   private long[] array;
 
   /**
@@ -65,16 +63,13 @@ final class LongArrayList extends AbstractProtobufList<Long>
    */
   private int size;
 
-  /**
-   * Constructs a new mutable {@code LongArrayList} with default capacity.
-   */
+  /** Constructs a new mutable {@code LongArrayList} with default capacity. */
   LongArrayList() {
     this(new long[DEFAULT_CAPACITY], 0);
   }
 
   /**
-   * Constructs a new mutable {@code LongArrayList}
-   * containing the same elements as {@code other}.
+   * Constructs a new mutable {@code LongArrayList} containing the same elements as {@code other}.
    */
   private LongArrayList(long[] other, int size) {
     array = other;
@@ -144,6 +139,26 @@ final class LongArrayList extends AbstractProtobufList<Long>
     return array[index];
   }
 
+  @Override
+  public int indexOf(Object element) {
+    if (!(element instanceof Long)) {
+      return -1;
+    }
+    long unboxedElement = (Long) element;
+    int numElems = size();
+    for (int i = 0; i < numElems; i++) {
+      if (array[i] == unboxedElement) {
+        return i;
+      }
+    }
+    return -1;
+  }
+
+  @Override
+  public boolean contains(Object element) {
+    return indexOf(element) != -1;
+  }
+
   @Override
   public int size() {
     return size;
@@ -163,22 +178,34 @@ final class LongArrayList extends AbstractProtobufList<Long>
     return previousValue;
   }
 
+  @Override
+  public boolean add(Long element) {
+    addLong(element);
+    return true;
+  }
+
   @Override
   public void add(int index, Long element) {
     addLong(index, element);
   }
 
-  /**
-   * Like {@link #add(Long)} but more efficient in that it doesn't box the element.
-   */
+  /** Like {@link #add(Long)} but more efficient in that it doesn't box the element. */
   @Override
   public void addLong(long element) {
-    addLong(size, element);
+    ensureIsMutable();
+    if (size == array.length) {
+      // Resize to 1.5x the size
+      int length = ((size * 3) / 2) + 1;
+      long[] newArray = new long[length];
+
+      System.arraycopy(array, 0, newArray, 0, size);
+      array = newArray;
+    }
+
+    array[size++] = element;
   }
 
-  /**
-   * Like {@link #add(int, Long)} but more efficient in that it doesn't box the element.
-   */
+  /** Like {@link #add(int, Long)} but more efficient in that it doesn't box the element. */
   private void addLong(int index, long element) {
     ensureIsMutable();
     if (index < 0 || index > size) {
@@ -244,7 +271,7 @@ final class LongArrayList extends AbstractProtobufList<Long>
     ensureIsMutable();
     for (int i = 0; i < size; i++) {
       if (o.equals(array[i])) {
-        System.arraycopy(array, i + 1, array, i, size - i);
+        System.arraycopy(array, i + 1, array, i, size - i - 1);
         size--;
         modCount++;
         return true;
@@ -259,7 +286,7 @@ final class LongArrayList extends AbstractProtobufList<Long>
     ensureIndexInRange(index);
     long value = array[index];
     if (index < size - 1) {
-      System.arraycopy(array, index + 1, array, index, size - index);
+      System.arraycopy(array, index + 1, array, index, size - index - 1);
     }
     size--;
     modCount++;
