@@ -37,9 +37,8 @@ import java.util.List;
 import junit.framework.TestCase;
 
 /**
- * Tests for {@link RepeatedFieldBuilderV3}. This tests basic functionality.
- * More extensive testing is provided via other tests that exercise the
- * builder.
+ * Tests for {@link RepeatedFieldBuilderV3}. This tests basic functionality. More extensive testing is
+ * provided via other tests that exercise the builder.
  *
  * @author jonp@google.com (Jon Perlow)
  */
@@ -47,8 +46,8 @@ public class RepeatedFieldBuilderV3Test extends TestCase {
 
   public void testBasicUse() {
     TestUtil.MockBuilderParent mockParent = new TestUtil.MockBuilderParent();
-    RepeatedFieldBuilderV3<TestAllTypes, TestAllTypes.Builder,
-        TestAllTypesOrBuilder> builder = newRepeatedFieldBuilderV3(mockParent);
+    RepeatedFieldBuilderV3<TestAllTypes, TestAllTypes.Builder, TestAllTypesOrBuilder> builder =
+        newRepeatedFieldBuilderV3(mockParent);
     builder.addMessage(TestAllTypes.newBuilder().setOptionalInt32(0).build());
     builder.addMessage(TestAllTypes.newBuilder().setOptionalInt32(1).build());
     assertEquals(0, builder.getMessage(0).getOptionalInt32());
@@ -68,8 +67,8 @@ public class RepeatedFieldBuilderV3Test extends TestCase {
 
   public void testGoingBackAndForth() {
     TestUtil.MockBuilderParent mockParent = new TestUtil.MockBuilderParent();
-    RepeatedFieldBuilderV3<TestAllTypes, TestAllTypes.Builder,
-        TestAllTypesOrBuilder> builder = newRepeatedFieldBuilderV3(mockParent);
+    RepeatedFieldBuilderV3<TestAllTypes, TestAllTypes.Builder, TestAllTypesOrBuilder> builder =
+        newRepeatedFieldBuilderV3(mockParent);
     builder.addMessage(TestAllTypes.newBuilder().setOptionalInt32(0).build());
     builder.addMessage(TestAllTypes.newBuilder().setOptionalInt32(1).build());
     assertEquals(0, builder.getMessage(0).getOptionalInt32());
@@ -89,7 +88,7 @@ public class RepeatedFieldBuilderV3Test extends TestCase {
     list = builder.build();
     assertEquals(2, list.size());
     assertEquals(0, list.get(0).getOptionalInt32());
-      assertEquals("foo", list.get(0).getOptionalString());
+    assertEquals("foo", list.get(0).getOptionalString());
     assertEquals(1, list.get(1).getOptionalInt32());
     assertIsUnmodifiable(list);
     assertEquals(1, mockParent.getInvalidationCount());
@@ -97,12 +96,11 @@ public class RepeatedFieldBuilderV3Test extends TestCase {
 
   public void testVariousMethods() {
     TestUtil.MockBuilderParent mockParent = new TestUtil.MockBuilderParent();
-    RepeatedFieldBuilderV3<TestAllTypes, TestAllTypes.Builder,
-        TestAllTypesOrBuilder> builder = newRepeatedFieldBuilderV3(mockParent);
+    RepeatedFieldBuilderV3<TestAllTypes, TestAllTypes.Builder, TestAllTypesOrBuilder> builder =
+        newRepeatedFieldBuilderV3(mockParent);
     builder.addMessage(TestAllTypes.newBuilder().setOptionalInt32(1).build());
     builder.addMessage(TestAllTypes.newBuilder().setOptionalInt32(2).build());
-    builder.addBuilder(0, TestAllTypes.getDefaultInstance())
-        .setOptionalInt32(0);
+    builder.addBuilder(0, TestAllTypes.getDefaultInstance()).setOptionalInt32(0);
     builder.addBuilder(TestAllTypes.getDefaultInstance()).setOptionalInt32(3);
 
     assertEquals(0, builder.getMessage(0).getOptionalInt32());
@@ -139,11 +137,10 @@ public class RepeatedFieldBuilderV3Test extends TestCase {
 
   public void testLists() {
     TestUtil.MockBuilderParent mockParent = new TestUtil.MockBuilderParent();
-    RepeatedFieldBuilderV3<TestAllTypes, TestAllTypes.Builder,
-        TestAllTypesOrBuilder> builder = newRepeatedFieldBuilderV3(mockParent);
+    RepeatedFieldBuilderV3<TestAllTypes, TestAllTypes.Builder, TestAllTypesOrBuilder> builder =
+        newRepeatedFieldBuilderV3(mockParent);
     builder.addMessage(TestAllTypes.newBuilder().setOptionalInt32(1).build());
-    builder.addMessage(0,
-        TestAllTypes.newBuilder().setOptionalInt32(0).build());
+    builder.addMessage(0, TestAllTypes.newBuilder().setOptionalInt32(0).build());
     assertEquals(0, builder.getMessage(0).getOptionalInt32());
     assertEquals(1, builder.getMessage(1).getOptionalInt32());
 
@@ -178,11 +175,9 @@ public class RepeatedFieldBuilderV3Test extends TestCase {
     }
   }
 
-  private RepeatedFieldBuilderV3<TestAllTypes, TestAllTypes.Builder,
-      TestAllTypesOrBuilder>
+  private RepeatedFieldBuilderV3<TestAllTypes, TestAllTypes.Builder, TestAllTypesOrBuilder>
       newRepeatedFieldBuilderV3(GeneratedMessage.BuilderParent parent) {
-    return new RepeatedFieldBuilderV3<TestAllTypes, TestAllTypes.Builder,
-        TestAllTypesOrBuilder>(Collections.<TestAllTypes>emptyList(), false,
-        parent, false);
+    return new RepeatedFieldBuilderV3<TestAllTypes, TestAllTypes.Builder, TestAllTypesOrBuilder>(
+        Collections.<TestAllTypes>emptyList(), false, parent, false);
   }
 }
