@@ -32,7 +32,6 @@
 
 goog.setTestOnly();
 
-goog.require('goog.json');
 goog.require('goog.testing.PropertyReplacer');
 goog.require('goog.testing.asserts');
 goog.require('goog.userAgent');
@@ -118,6 +117,9 @@ goog.require('proto.jspb.test.TestReservedNamesExtension');
 goog.require('proto.jspb.test.ExtensionMessage');
 goog.require('proto.jspb.test.TestExtensionsMessage');
 
+goog.require('proto.jspb.test.TestAllowAliasEnum');
+// CommonJS-LoadFromFile: testlargenumbers_pb proto.jspb.test
+goog.require('proto.jspb.test.MessageWithLargeFieldNumbers');
 
 describe('Message test suite', function() {
   var stubs = new goog.testing.PropertyReplacer();
@@ -855,7 +857,7 @@ describe('Message test suite', function() {
     var assertNan = function(x) {
       assertTrue(
           'Expected ' + x + ' (' + goog.typeOf(x) + ') to be NaN.',
-          goog.isNumber(x) && isNaN(x));
+          typeof x === 'number' && isNaN(x));
     };
 
     var message = new proto.jspb.test.FloatingPointFields([
@@ -1075,4 +1077,36 @@ describe('Message test suite', function() {
     assertEquals(12, package2Message.getA());
   });
 
+
+  it('testMessageWithLargeFieldNumbers', function() {
+    var message = new proto.jspb.test.MessageWithLargeFieldNumbers;
+
+    message.setAString('string');
+    assertEquals('string', message.getAString());
+
+    message.setAString('');
+    assertEquals('', message.getAString());
+
+    message.setAString('new string');
+    assertEquals('new string', message.getAString());
+
+    message.setABoolean(true);
+    assertEquals(true, message.getABoolean());
+
+    message.setABoolean(false);
+    assertEquals(false, message.getABoolean());
+
+    message.setABoolean(true);
+    assertEquals(true, message.getABoolean());
+
+    message.setAInt(42);
+    assertEquals(42, message.getAInt());
+
+    message.setAInt(0);
+    assertEquals(0, message.getAInt());
+
+    message.setAInt(42);
+    assertEquals(42, message.getAInt());
+  });
+
 });
