@@ -2,6 +2,8 @@
 #
 # Build file to set up and run tests
 
+set -ex  # exit immediately on error
+
 # Change to repo root
 cd $(dirname $0)/../../..
 
@@ -9,7 +11,7 @@ cd $(dirname $0)/../../..
 
 # Run tests under release docker image.
 DOCKER_IMAGE_NAME=protobuf/protoc_$(sha1sum protoc-artifacts/Dockerfile | cut -f1 -d " ")
-docker pull $DOCKER_IMAGE_NAME
+until docker pull $DOCKER_IMAGE_NAME; do sleep 10; done
 
 docker run -v $(pwd):/var/local/protobuf --rm $DOCKER_IMAGE_NAME \
   bash -l /var/local/protobuf/tests.sh cpp || FAILED="true"
