@@ -28,7 +28,11 @@
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
+#include <php.h>
+#include <Zend/zend_exceptions.h>
+
 #include "protobuf.h"
+#include "builtin_descriptors.inc"
 
 // Forward declare.
 static void descriptor_init_c_instance(Descriptor* intern TSRMLS_DC);
@@ -65,53 +69,6 @@ static void check_upb_status(const upb_status* status, const char* msg) {
   }
 }
 
-static void upb_filedef_free(void *r) {
-  upb_filedef *f = *(upb_filedef **)r;
-  size_t i;
-
-  for (i = 0; i < upb_filedef_depcount(f); i++) {
-    upb_filedef_unref(upb_filedef_dep(f, i), f);
-  }
-
-  upb_inttable_uninit(&f->defs);
-  upb_inttable_uninit(&f->deps);
-  upb_gfree((void *)f->name);
-  upb_gfree((void *)f->package);
-  upb_gfree(f);
-}
-
-// Camel-case the field name and append "Entry" for generated map entry name.
-// e.g. map<KeyType, ValueType> foo_map => FooMapEntry
-static void append_map_entry_name(char *result, const char *field_name,
-                                  int pos) {
-  bool cap_next = true;
-  int i;
-
-  for (i = 0; i < strlen(field_name); ++i) {
-    if (field_name[i] == '_') {
-      cap_next = true;
-    } else if (cap_next) {
-      // Note: Do not use ctype.h due to locales.
-      if ('a' <= field_name[i] && field_name[i] <= 'z') {
-        result[pos++] = field_name[i] - 'a' + 'A';
-      } else {
-        result[pos++] = field_name[i];
-      }
-      cap_next = false;
-    } else {
-      result[pos++] = field_name[i];
-    }
-  }
-  strcat(result, "Entry");
-}
-
-#define CHECK_UPB(code, msg)             \
-  do {                                   \
-    upb_status status = UPB_STATUS_INIT; \
-    code;                                \
-    check_upb_status(&status, msg);      \
-  } while (0)
-
 // -----------------------------------------------------------------------------
 // GPBType
 // -----------------------------------------------------------------------------
@@ -166,47 +123,16 @@ static zend_function_entry descriptor_methods[] = {
 DEFINE_CLASS(Descriptor, descriptor, "Google\\Protobuf\\Descriptor");
 
 static void descriptor_free_c(Descriptor *self TSRMLS_DC) {
-  if (self->layout) {
-    free_layout(self->layout);
-  }
-  if (self->fill_handlers) {
-    upb_handlers_unref(self->fill_handlers, &self->fill_handlers);
-  }
-  if (self->fill_method) {
-    upb_pbdecodermethod_unref(self->fill_method, &self->fill_method);
-  }
-  if (self->json_fill_method) {
-    upb_json_parsermethod_unref(self->json_fill_method,
-                                &self->json_fill_method);
-  }
-  if (self->pb_serialize_handlers) {
-    upb_handlers_unref(self->pb_serialize_handlers,
-                       &self->pb_serialize_handlers);
-  }
-  if (self->json_serialize_handlers) {
-    upb_handlers_unref(self->json_serialize_handlers,
-                       &self->json_serialize_handlers);
-  }
-  if (self->json_serialize_handlers_preserve) {
-    upb_handlers_unref(self->json_serialize_handlers_preserve,
-                       &self->json_serialize_handlers_preserve);
-  }
 }
 
 static void descriptor_init_c_instance(Descriptor *desc TSRMLS_DC) {
-  desc->msgdef = NULL;
-  desc->layout = NULL;
-  desc->klass = NULL;
-  desc->fill_handlers = NULL;
-  desc->fill_method = NULL;
-  desc->json_fill_method = NULL;
-  desc->pb_serialize_handlers = NULL;
-  desc->json_serialize_handlers = NULL;
-  desc->json_serialize_handlers_preserve = NULL;
+  desc->intern = NULL;
 }
 
 PHP_METHOD(Descriptor, getClass) {
-  Descriptor *intern = UNBOX(Descriptor, getThis());
+  Descriptor* desc = UNBOX(Descriptor, getThis());
+  DescriptorInternal* intern = desc->intern;
+  register_class(intern, false TSRMLS_CC);
 #if PHP_MAJOR_VERSION < 7
   const char* classname = intern->klass->name;
 #else
@@ -216,7 +142,8 @@ PHP_METHOD(Descriptor, getClass) {
 }
 
 PHP_METHOD(Descriptor, getFullName) {
-  Descriptor *intern = UNBOX(Descriptor, getThis());
+  Descriptor* desc = UNBOX(Descriptor, getThis());
+  DescriptorInternal* intern = desc->intern;
   const char* fullname = upb_msgdef_fullname(intern->msgdef);
   PHP_PROTO_RETVAL_STRINGL(fullname, strlen(fullname), 1);
 }
@@ -229,7 +156,8 @@ PHP_METHOD(Descriptor, getField) {
     return;
   }
 
-  Descriptor *intern = UNBOX(Descriptor, getThis());
+  Descriptor* desc = UNBOX(Descriptor, getThis());
+  DescriptorInternal* intern = desc->intern;
   int field_num = upb_msgdef_numfields(intern->msgdef);
   if (index < 0 || index >= field_num) {
     zend_error(E_USER_ERROR, "Cannot get element at %ld.\n", index);
@@ -270,7 +198,8 @@ PHP_METHOD(Descriptor, getField) {
 }
 
 PHP_METHOD(Descriptor, getFieldCount) {
-  Descriptor *intern = UNBOX(Descriptor, getThis());
+  Descriptor* desc = UNBOX(Descriptor, getThis());
+  DescriptorInternal* intern = desc->intern;
   RETURN_LONG(upb_msgdef_numfields(intern->msgdef));
 }
 
@@ -282,7 +211,8 @@ PHP_METHOD(Descriptor, getOneofDecl) {
     return;
   }
 
-  Descriptor *intern = UNBOX(Descriptor, getThis());
+  Descriptor* desc = UNBOX(Descriptor, getThis());
+  DescriptorInternal* intern = desc->intern;
   int field_num = upb_msgdef_numoneofs(intern->msgdef);
   if (index < 0 || index >= field_num) {
     zend_error(E_USER_ERROR, "Cannot get element at %ld.\n", index);
@@ -294,7 +224,7 @@ PHP_METHOD(Descriptor, getOneofDecl) {
   for(upb_msg_oneof_begin(&iter, intern->msgdef), i = 0;
       !upb_msg_oneof_done(&iter) && i < index;
       upb_msg_oneof_next(&iter), i++);
-  upb_oneofdef *oneof = upb_msg_iter_oneof(&iter);
+  const upb_oneofdef *oneof = upb_msg_iter_oneof(&iter);
 
   ZVAL_OBJ(return_value, oneof_descriptor_type->create_object(
                              oneof_descriptor_type TSRMLS_CC));
@@ -303,7 +233,8 @@ PHP_METHOD(Descriptor, getOneofDecl) {
 }
 
 PHP_METHOD(Descriptor, getOneofDeclCount) {
-  Descriptor *intern = UNBOX(Descriptor, getThis());
+  Descriptor* desc = UNBOX(Descriptor, getThis());
+  DescriptorInternal* intern = desc->intern;
   RETURN_LONG(upb_msgdef_numoneofs(intern->msgdef));
 }
 
@@ -324,8 +255,7 @@ static void enum_descriptor_free_c(EnumDescriptor *self TSRMLS_DC) {
 }
 
 static void enum_descriptor_init_c_instance(EnumDescriptor *self TSRMLS_DC) {
-  self->enumdef = NULL;
-  self->klass = NULL;
+  self->intern = NULL;
 }
 
 PHP_METHOD(EnumDescriptor, getValue) {
@@ -336,7 +266,8 @@ PHP_METHOD(EnumDescriptor, getValue) {
     return;
   }
 
-  EnumDescriptor *intern = UNBOX(EnumDescriptor, getThis());
+  EnumDescriptor *desc = UNBOX(EnumDescriptor, getThis());
+  EnumDescriptorInternal *intern = desc->intern;
   int field_num = upb_enumdef_numvals(intern->enumdef);
   if (index < 0 || index >= field_num) {
     zend_error(E_USER_ERROR, "Cannot get element at %ld.\n", index);
@@ -358,7 +289,8 @@ PHP_METHOD(EnumDescriptor, getValue) {
 }
 
 PHP_METHOD(EnumDescriptor, getValueCount) {
-  EnumDescriptor *intern = UNBOX(EnumDescriptor, getThis());
+  EnumDescriptor *desc = UNBOX(EnumDescriptor, getThis());
+  EnumDescriptorInternal *intern = desc->intern;
   RETURN_LONG(upb_enumdef_numvals(intern->enumdef));
 }
 
@@ -479,41 +411,77 @@ PHP_METHOD(FieldDescriptor, isMap) {
 
 PHP_METHOD(FieldDescriptor, getEnumType) {
   FieldDescriptor *intern = UNBOX(FieldDescriptor, getThis());
-  const upb_enumdef *enumdef = upb_fielddef_enumsubdef(intern->fielddef);
-  if (enumdef == NULL) {
-    char error_msg[100];
-    sprintf(error_msg, "Cannot get enum type for non-enum field '%s'",
-            upb_fielddef_name(intern->fielddef));
-    zend_throw_exception(NULL, error_msg, 0 TSRMLS_CC);
+  if (upb_fielddef_type(intern->fielddef) != UPB_TYPE_ENUM) {
+    zend_throw_exception_ex(NULL, 0 TSRMLS_CC,
+                            "Cannot get enum type for non-enum field '%s'",
+                            upb_fielddef_name(intern->fielddef));
     return;
   }
-  PHP_PROTO_HASHTABLE_VALUE desc = get_def_obj(enumdef);
+  const upb_enumdef *enumdef = upb_fielddef_enumsubdef(intern->fielddef);
+  PHP_PROTO_HASHTABLE_VALUE desc_php = get_def_obj(enumdef);
+
+  if (desc_php == NULL) {
+    EnumDescriptorInternal* intern = get_enumdef_enumdesc(enumdef);
 
 #if PHP_MAJOR_VERSION < 7
-  RETURN_ZVAL(desc, 1, 0);
+    MAKE_STD_ZVAL(desc_php);
+    ZVAL_OBJ(desc_php, enum_descriptor_type->create_object(
+                                        enum_descriptor_type TSRMLS_CC));
+    Z_DELREF_P(desc_php);
 #else
-  GC_ADDREF(desc);
-  RETURN_OBJ(desc);
+    desc_php =
+        enum_descriptor_type->create_object(enum_descriptor_type TSRMLS_CC);
+    GC_DELREF(desc_php);
+#endif
+    EnumDescriptor* desc = UNBOX_HASHTABLE_VALUE(EnumDescriptor, desc_php);
+    desc->intern = intern;
+    add_def_obj(enumdef, desc_php);
+    add_ce_obj(intern->klass, desc_php);
+  }
+
+#if PHP_MAJOR_VERSION < 7
+  RETURN_ZVAL(desc_php, 1, 0);
+#else
+  GC_ADDREF(desc_php);
+  RETURN_OBJ(desc_php);
 #endif
 }
 
 PHP_METHOD(FieldDescriptor, getMessageType) {
   FieldDescriptor *intern = UNBOX(FieldDescriptor, getThis());
-  const upb_msgdef *msgdef = upb_fielddef_msgsubdef(intern->fielddef);
-  if (msgdef == NULL) {
-    char error_msg[100];
-    sprintf(error_msg, "Cannot get message type for non-message field '%s'",
-            upb_fielddef_name(intern->fielddef));
-    zend_throw_exception(NULL, error_msg, 0 TSRMLS_CC);
+  if (upb_fielddef_type(intern->fielddef) != UPB_TYPE_MESSAGE) {
+    zend_throw_exception_ex(
+        NULL, 0 TSRMLS_CC, "Cannot get message type for non-message field '%s'",
+        upb_fielddef_name(intern->fielddef));
     return;
   }
-  PHP_PROTO_HASHTABLE_VALUE desc = get_def_obj(msgdef);
+  const upb_msgdef *msgdef = upb_fielddef_msgsubdef(intern->fielddef);
+  PHP_PROTO_HASHTABLE_VALUE desc_php = get_def_obj(msgdef);
+
+  if (desc_php == NULL) {
+    DescriptorInternal* intern = get_msgdef_desc(msgdef);
 
 #if PHP_MAJOR_VERSION < 7
-  RETURN_ZVAL(desc, 1, 0);
+    MAKE_STD_ZVAL(desc_php);
+    ZVAL_OBJ(desc_php, descriptor_type->create_object(
+                                   descriptor_type TSRMLS_CC));
+    Z_DELREF_P(desc_php);
 #else
-  GC_ADDREF(desc);
-  RETURN_OBJ(desc);
+    desc_php =
+        descriptor_type->create_object(descriptor_type TSRMLS_CC);
+    GC_DELREF(desc_php);
+#endif
+    Descriptor* desc = UNBOX_HASHTABLE_VALUE(Descriptor, desc_php);
+    desc->intern = intern;
+    add_def_obj(msgdef, desc_php);
+    add_ce_obj(intern->klass, desc_php);
+  }
+
+#if PHP_MAJOR_VERSION < 7
+  RETURN_ZVAL(desc_php, 1, 0);
+#else
+  GC_ADDREF(desc_php);
+  RETURN_OBJ(desc_php);
 #endif
 }
 
@@ -627,7 +595,8 @@ zval* internal_generated_pool_php;
 zend_object *generated_pool_php;
 zend_object *internal_generated_pool_php;
 #endif
-InternalDescriptorPool *generated_pool;  // The actual generated pool
+InternalDescriptorPoolImpl *generated_pool;
+InternalDescriptorPoolImpl generated_pool_impl;  // The actual generated pool
 
 void init_generated_pool_once(TSRMLS_D) {
   if (generated_pool == NULL) {
@@ -637,34 +606,48 @@ void init_generated_pool_once(TSRMLS_D) {
     ZVAL_OBJ(internal_generated_pool_php,
              internal_descriptor_pool_type->create_object(
                  internal_descriptor_pool_type TSRMLS_CC));
-    generated_pool = UNBOX(InternalDescriptorPool, internal_generated_pool_php);
     ZVAL_OBJ(generated_pool_php, descriptor_pool_type->create_object(
                                      descriptor_pool_type TSRMLS_CC));
 #else
     internal_generated_pool_php = internal_descriptor_pool_type->create_object(
         internal_descriptor_pool_type TSRMLS_CC);
-    generated_pool = (InternalDescriptorPool *)((char *)internal_generated_pool_php -
-                                        XtOffsetOf(InternalDescriptorPool, std));
     generated_pool_php =
         descriptor_pool_type->create_object(descriptor_pool_type TSRMLS_CC);
 #endif
+    generated_pool = &generated_pool_impl;
   }
 }
 
 static void internal_descriptor_pool_init_c_instance(
     InternalDescriptorPool *pool TSRMLS_DC) {
-  pool->symtab = upb_symtab_new();
-
-  ALLOC_HASHTABLE(pool->pending_list);
-  zend_hash_init(pool->pending_list, 1, NULL, ZVAL_PTR_DTOR, 0);
+  pool->intern = &generated_pool_impl;
 }
 
 static void internal_descriptor_pool_free_c(
     InternalDescriptorPool *pool TSRMLS_DC) {
-  upb_symtab_free(pool->symtab);
+}
 
-  zend_hash_destroy(pool->pending_list);
-  FREE_HASHTABLE(pool->pending_list);
+void internal_descriptor_pool_impl_init(
+    InternalDescriptorPoolImpl *pool TSRMLS_DC) {
+  pool->symtab = upb_symtab_new();
+  pool->fill_handler_cache =
+      upb_handlercache_new(add_handlers_for_message, NULL);
+  pool->pb_serialize_handler_cache = upb_pb_encoder_newcache();
+  pool->json_serialize_handler_cache = upb_json_printer_newcache(false);
+  pool->json_serialize_handler_preserve_cache = upb_json_printer_newcache(true);
+  pool->fill_method_cache = upb_pbcodecache_new(pool->fill_handler_cache);
+  pool->json_fill_method_cache = upb_json_codecache_new();
+}
+
+void internal_descriptor_pool_impl_destroy(
+    InternalDescriptorPoolImpl *pool TSRMLS_DC) {
+  upb_symtab_free(pool->symtab);
+  upb_handlercache_free(pool->fill_handler_cache);
+  upb_handlercache_free(pool->pb_serialize_handler_cache);
+  upb_handlercache_free(pool->json_serialize_handler_cache);
+  upb_handlercache_free(pool->json_serialize_handler_preserve_cache);
+  upb_pbcodecache_free(pool->fill_method_cache);
+  upb_json_codecache_free(pool->json_fill_method_cache);
 }
 
 static void descriptor_pool_init_c_instance(DescriptorPool *pool TSRMLS_DC) {
@@ -675,29 +658,6 @@ static void descriptor_pool_init_c_instance(DescriptorPool *pool TSRMLS_DC) {
 static void descriptor_pool_free_c(DescriptorPool *pool TSRMLS_DC) {
 }
 
-static void validate_enumdef(const upb_enumdef *enumdef) {
-  // Verify that an entry exists with integer value 0. (This is the default
-  // value.)
-  const char *lookup = upb_enumdef_iton(enumdef, 0);
-  if (lookup == NULL) {
-    zend_error(E_USER_ERROR,
-               "Enum definition does not contain a value for '0'.");
-  }
-}
-
-static void validate_msgdef(const upb_msgdef* msgdef) {
-  // Verify that no required fields exist. proto3 does not support these.
-  upb_msg_field_iter it;
-  for (upb_msg_field_begin(&it, msgdef);
-       !upb_msg_field_done(&it);
-       upb_msg_field_next(&it)) {
-    const upb_fielddef* field = upb_msg_iter_field(&it);
-    if (upb_fielddef_label(field) == UPB_LABEL_REQUIRED) {
-      zend_error(E_ERROR, "Required fields are unsupported in proto3.");
-    }
-  }
-}
-
 PHP_METHOD(DescriptorPool, getGeneratedPool) {
   init_generated_pool_once(TSRMLS_C);
 #if PHP_MAJOR_VERSION < 7
@@ -718,58 +678,6 @@ PHP_METHOD(InternalDescriptorPool, getGeneratedPool) {
 #endif
 }
 
-static size_t classname_len_max(const char *fullname,
-                                const char *package,
-                                const char *php_namespace,
-                                const char *prefix) {
-  size_t fullname_len = strlen(fullname);
-  size_t package_len = 0;
-  size_t prefix_len = 0;
-  size_t namespace_len = 0;
-  size_t length = fullname_len;
-  int i, segment, classname_start = 0;
-
-  if (package != NULL) {
-    package_len = strlen(package);
-  }
-  if (prefix != NULL) {
-    prefix_len = strlen(prefix);
-  }
-  if (php_namespace != NULL) {
-    namespace_len = strlen(php_namespace);
-  }
-
-  // Process package
-  if (package_len > 0) {
-    segment = 1;
-    for (i = 0; i < package_len; i++) {
-      if (package[i] == '.') {
-        segment++;
-      }
-    }
-    // In case of reserved name in package.
-    length += 3 * segment;
-
-    classname_start = package_len + 1;
-  }
-
-  // Process class name
-  segment = 1;
-  for (i = classname_start; i < fullname_len; i++) {
-    if (fullname[i] == '.') {
-      segment++;
-    }
-  }
-  if (prefix_len == 0) {
-    length += 3 * segment;
-  } else {
-    length += prefix_len * segment;
-  }
-
-  // The additional 2, one is for preceding '.' and the other is for trailing 0.
-  return length + namespace_len + 2;
-}
-
 static bool is_reserved(const char *segment, int length) {
   bool result;
   char* lower = ALLOC_N(char, length + 1);
@@ -786,78 +694,72 @@ static bool is_reserved(const char *segment, int length) {
   return result;
 }
 
-static char* fill_prefix(const char *segment, int length,
-                         const char *prefix_given,
-                         const char *package_name, char *classname) {
-  size_t i;
-
+static void fill_prefix(const char *segment, int length,
+                        const char *prefix_given,
+                        const char *package_name,
+                        stringsink *classname) {
   if (prefix_given != NULL && strcmp(prefix_given, "") != 0) {
-    size_t prefix_len = strlen(prefix_given);
-    memcpy(classname, prefix_given, strlen(prefix_given));
-    classname += prefix_len;
+    stringsink_string(classname, NULL, prefix_given,
+                      strlen(prefix_given), NULL);
   } else {
     if (is_reserved(segment, length)) {
       if (package_name != NULL &&
           strcmp("google.protobuf", package_name) == 0) {
-        memcpy(classname, "GPB", 3);
-        classname += 3;
+        stringsink_string(classname, NULL, "GPB", 3, NULL);
       } else {
-        memcpy(classname, "PB", 2);
-        classname += 2;
+        stringsink_string(classname, NULL, "PB", 2, NULL);
       }
     }
   }
-  return classname;
 }
 
-static char* fill_segment(const char *segment, int length,
-                          char *classname, bool use_camel) {
-  memcpy(classname, segment, length);
+static void fill_segment(const char *segment, int length,
+                         stringsink *classname, bool use_camel) {
   if (use_camel && (segment[0] < 'A' || segment[0] > 'Z')) {
-    classname[0] += 'A' - 'a';
+    char first = segment[0] + ('A' - 'a');
+    stringsink_string(classname, NULL, &first, 1, NULL);
+    stringsink_string(classname, NULL, segment + 1, length - 1, NULL);
+  } else {
+    stringsink_string(classname, NULL, segment, length, NULL);
   }
-  return classname + length;
 }
 
-static char* fill_namespace(const char *package, const char *namespace_given,
-                            char *classname) {
-  if (namespace_given != NULL) {
-    size_t namespace_len = strlen(namespace_given);
-    memcpy(classname, namespace_given, namespace_len);
-    classname += namespace_len;
-    *classname = '\\';
-    classname++;
+static void fill_namespace(const char *package, const char *php_namespace,
+                           stringsink *classname) {
+  if (php_namespace != NULL) {
+    if (strlen(php_namespace) != 0) {
+      stringsink_string(classname, NULL, php_namespace, strlen(php_namespace),
+                        NULL);
+      stringsink_string(classname, NULL, "\\", 1, NULL);
+    }
   } else if (package != NULL) {
-    int i = 0, j, offset = 0;
+    int i = 0, j = 0;
     size_t package_len = strlen(package);
     while (i < package_len) {
       j = i;
       while (j < package_len && package[j] != '.') {
         j++;
       }
-      classname = fill_prefix(package + i, j - i, "", package, classname);
-      classname = fill_segment(package + i, j - i, classname, true);
-      classname[0] = '\\';
-      classname++;
+      fill_prefix(package + i, j - i, "", package, classname);
+      fill_segment(package + i, j - i, classname, true);
+      stringsink_string(classname, NULL, "\\", 1, NULL);
       i = j + 1;
     }
   }
-  return classname;
 }
 
-static char* fill_classname(const char *fullname,
-                            const char *package,
-                            const char *namespace_given,
-                            const char *prefix, char *classname) {
+static void fill_classname(const char *fullname,
+                           const char *package,
+                           const char *prefix,
+                           stringsink *classname,
+                           bool use_nested_submsg) {
   int classname_start = 0;
   if (package != NULL) {
     size_t package_len = strlen(package);
     classname_start = package_len == 0 ? 0 : package_len + 1;
   }
   size_t fullname_len = strlen(fullname);
-  classname = fill_prefix(fullname + classname_start,
-                          fullname_len - classname_start,
-                          prefix, package, classname);
+  bool is_first_segment = true;
 
   int i = classname_start, j;
   while (i < fullname_len) {
@@ -865,146 +767,264 @@ static char* fill_classname(const char *fullname,
     while (j < fullname_len && fullname[j] != '.') {
       j++;
     }
-    classname = fill_segment(fullname + i, j - i, classname, false);
+    if (use_nested_submsg || (is_first_segment && j == fullname_len)) {
+      fill_prefix(fullname + i, j - i, prefix, package, classname);
+    }
+    is_first_segment = false;
+    fill_segment(fullname + i, j - i, classname, false);
     if (j != fullname_len) {
-      *classname = '_';
-      classname++;
+      if (use_nested_submsg) {
+        stringsink_string(classname, NULL, "\\", 1, NULL);
+      } else {
+        stringsink_string(classname, NULL, "_", 1, NULL);
+      }
     }
     i = j + 1;
   }
-  return classname;
 }
 
-static char* fill_qualified_classname(const char *fullname,
-                                      const char *package,
-                                      const char *namespace_given,
-                                      const char *prefix, char *classname) {
-  classname = fill_namespace(package, namespace_given, classname);
-  return fill_classname(fullname, package, namespace_given, prefix, classname);
+static void fill_classname_for_desc(void *desc, bool is_enum) {
+  const upb_filedef *file;
+  const char *fullname;
+  bool use_nested_submsg;
+
+  if (is_enum) {
+    EnumDescriptorInternal* enumdesc = desc;
+    file = upb_enumdef_file(enumdesc->enumdef);
+    fullname = upb_enumdef_fullname(enumdesc->enumdef);
+    use_nested_submsg = enumdesc->use_nested_submsg;
+  } else {
+    DescriptorInternal* msgdesc = desc;
+    file = upb_msgdef_file(msgdesc->msgdef);
+    fullname = upb_msgdef_fullname(msgdesc->msgdef);
+    use_nested_submsg = msgdesc->use_nested_submsg;
+  }
+
+  // Prepend '.' to package name to make it absolute. In the 5 additional
+  // bytes allocated, one for '.', one for trailing 0, and 3 for 'GPB' if
+  // given message is google.protobuf.Empty.
+  const char *package = upb_filedef_package(file);
+  const char *php_namespace = upb_filedef_phpnamespace(file);
+  const char *prefix = upb_filedef_phpprefix(file);
+  stringsink namesink;
+  stringsink_init(&namesink);
+
+  fill_namespace(package, php_namespace, &namesink);
+  fill_classname(fullname, package, prefix, &namesink, use_nested_submsg);
+  stringsink_string(&namesink, NULL, "\0", 1, NULL);
+
+  if (is_enum) {
+    EnumDescriptorInternal* enumdesc = desc;
+    enumdesc->classname = strdup(namesink.ptr);
+  } else {
+    DescriptorInternal* msgdesc = desc;
+    msgdesc->classname = strdup(namesink.ptr);
+  }
+
+  stringsink_uninit(&namesink);
 }
 
-static void classname_no_prefix(const char *fullname, const char *package_name,
-                                char *class_name) {
-  size_t i = 0, j;
-  bool first_char = true, is_reserved = false;
-  size_t pkg_name_len = package_name == NULL ? 0 : strlen(package_name);
-  size_t message_name_start = package_name == NULL ? 0 : pkg_name_len + 1;
-  size_t message_len = (strlen(fullname) - message_name_start);
+void register_class(void *desc, bool is_enum TSRMLS_DC) {
+  const char *classname;
+  const char *fullname;
+  zend_class_entry* ret;
 
-  // Submessage is concatenated with its containing messages by '_'.
-  for (j = message_name_start; j < message_name_start + message_len; j++) {
-    if (fullname[j] == '.') {
-      class_name[i++] = '_';
-    } else {
-      class_name[i++] = fullname[j];
+  if (is_enum) {
+    EnumDescriptorInternal* enumdesc = desc;
+    if (enumdesc->klass) {
+      return;
+    }
+    classname = enumdesc->classname;
+    fullname = upb_enumdef_fullname(enumdesc->enumdef);
+  } else {
+    DescriptorInternal* msgdesc = desc;
+    if (msgdesc->klass) {
+      return;
+    }
+    if (!msgdesc->classname) {
+      return;
+    }
+    classname = msgdesc->classname;
+    fullname = upb_msgdef_fullname(msgdesc->msgdef);
+  }
+
+  PHP_PROTO_CE_DECLARE pce;
+  if (php_proto_zend_lookup_class(classname, strlen(classname), &pce) ==
+      FAILURE) {
+    zend_error(
+        E_ERROR,
+        "Generated message class %s hasn't been defined (%s)",
+        classname, fullname);
+    return;
+  }
+  ret = PHP_PROTO_CE_UNREF(pce);
+  if (is_enum) {
+    EnumDescriptorInternal* enumdesc = desc;
+    add_ce_enumdesc(ret, desc);
+    enumdesc->klass = ret;
+  } else {
+    DescriptorInternal* msgdesc = desc;
+    add_ce_desc(ret, desc);
+    msgdesc->klass = ret;
+    // Map entries don't have existing php class.
+    if (!upb_msgdef_mapentry(msgdesc->msgdef)) {
+      if (msgdesc->layout == NULL) {
+        MessageLayout* layout = create_layout(msgdesc->msgdef);
+        msgdesc->layout = layout;
+      }
     }
   }
 }
 
-void internal_add_generated_file(const char *data, PHP_PROTO_SIZE data_len,
-                                 InternalDescriptorPool *pool TSRMLS_DC) {
-  upb_filedef **files;
+bool depends_on_descriptor(const google_protobuf_FileDescriptorProto* file) {
+  const upb_strview *deps;
+  upb_strview name = upb_strview_makez("google/protobuf/descriptor.proto");
+  size_t i, n;
+
+  deps = google_protobuf_FileDescriptorProto_dependency(file, &n);
+  for (i = 0; i < n; i++) {
+    if (upb_strview_eql(deps[i], name)) {
+      return true;
+    }
+  }
+
+  return false;
+}
+
+static void internal_add_single_generated_file(
+    const upb_filedef* file,
+    InternalDescriptorPoolImpl* pool,
+    bool use_nested_submsg TSRMLS_DC) {
   size_t i;
-
-  CHECK_UPB(files = upb_loaddescriptor(data, data_len, &pool, &status),
-            "Parse binary descriptors to internal descriptors failed");
-
-  // This method is called only once in each file.
-  assert(files[0] != NULL);
-  assert(files[1] == NULL);
-
-  CHECK_UPB(upb_symtab_addfile(pool->symtab, files[0], &status),
-            "Unable to add file to DescriptorPool");
-
   // For each enum/message, we need its PHP class, upb descriptor and its PHP
   // wrapper. These information are needed later for encoding, decoding and type
   // checking. However, sometimes we just have one of them. In order to find
   // them quickly, here, we store the mapping for them.
-  for (i = 0; i < upb_filedef_defcount(files[0]); i++) {
-    const upb_def *def = upb_filedef_def(files[0], i);
-    switch (upb_def_type(def)) {
-#define CASE_TYPE(def_type, def_type_lower, desc_type, desc_type_lower)        \
-  case UPB_DEF_##def_type: {                                                   \
-    CREATE_HASHTABLE_VALUE(desc, desc_php, desc_type, desc_type_lower##_type); \
-    const upb_##def_type_lower *def_type_lower =                               \
-        upb_downcast_##def_type_lower(def);                                    \
-    desc->def_type_lower = def_type_lower;                                     \
-    add_def_obj(desc->def_type_lower, desc_php);                               \
-    /* Unlike other messages, MapEntry is shared by all map fields and doesn't \
-     * have generated PHP class.*/                                             \
-    if (upb_def_type(def) == UPB_DEF_MSG &&                                    \
-        upb_msgdef_mapentry(upb_downcast_msgdef(def))) {                       \
-      break;                                                                   \
-    }                                                                          \
-    /* Prepend '.' to package name to make it absolute. In the 5 additional    \
-     * bytes allocated, one for '.', one for trailing 0, and 3 for 'GPB' if    \
-     * given message is google.protobuf.Empty.*/                               \
-    const char *fullname = upb_##def_type_lower##_fullname(def_type_lower);    \
-    const char *package = upb_filedef_package(files[0]);                       \
-    const char *php_namespace = upb_filedef_phpnamespace(files[0]);            \
-    const char *prefix_given = upb_filedef_phpprefix(files[0]);                \
-    size_t classname_len = classname_len_max(fullname, package,                \
-                                             php_namespace, prefix_given);     \
-    char *classname = ecalloc(sizeof(char), classname_len);                    \
-    fill_qualified_classname(fullname, package, php_namespace,                 \
-                             prefix_given, classname);                         \
-    PHP_PROTO_CE_DECLARE pce;                                                  \
-    if (php_proto_zend_lookup_class(classname, strlen(classname), &pce) ==     \
-        FAILURE) {                                                             \
-      zend_error(E_ERROR, "Generated message class %s hasn't been defined",    \
-                 classname);                                                   \
-      return;                                                                  \
-    } else {                                                                   \
-      desc->klass = PHP_PROTO_CE_UNREF(pce);                                   \
-    }                                                                          \
-    add_ce_obj(desc->klass, desc_php);                                         \
-    add_proto_obj(upb_##def_type_lower##_fullname(desc->def_type_lower),       \
-                  desc_php);                                                   \
-    efree(classname);                                                          \
-    break;                                                                     \
-  }
+  for (i = 0; i < upb_filedef_msgcount(file); i++) {
+    const upb_msgdef *msgdef = upb_filedef_msg(file, i);
+    CREATE_HASHTABLE_VALUE(desc, desc_php, Descriptor, descriptor_type);
+    desc->intern = SYS_MALLOC(DescriptorInternal);
+    desc->intern->msgdef = msgdef;
+    desc->intern->pool = pool;
+    desc->intern->layout = NULL;
+    desc->intern->klass = NULL;
+    desc->intern->use_nested_submsg = use_nested_submsg;
+    desc->intern->classname = NULL;
 
-      CASE_TYPE(MSG, msgdef, Descriptor, descriptor)
-      CASE_TYPE(ENUM, enumdef, EnumDescriptor, enum_descriptor)
-#undef CASE_TYPE
+    add_def_obj(desc->intern->msgdef, desc_php);
+    add_msgdef_desc(desc->intern->msgdef, desc->intern);
 
-      default:
-        break;
+    // Unlike other messages, MapEntry is shared by all map fields and doesn't
+    // have generated PHP class.
+    if (upb_msgdef_mapentry(msgdef)) {
+      continue;
     }
+
+    fill_classname_for_desc(desc->intern, false);
+    add_class_desc(desc->intern->classname, desc->intern);
+    add_proto_desc(upb_msgdef_fullname(desc->intern->msgdef), desc->intern);
   }
 
-  for (i = 0; i < upb_filedef_defcount(files[0]); i++) {
-    const upb_def *def = upb_filedef_def(files[0], i);
-    if (upb_def_type(def) == UPB_DEF_MSG) {
-      const upb_msgdef *msgdef = upb_downcast_msgdef(def);
-      PHP_PROTO_HASHTABLE_VALUE desc_php = get_def_obj(msgdef);
-      build_class_from_descriptor(desc_php TSRMLS_CC);
+  for (i = 0; i < upb_filedef_enumcount(file); i++) {
+    const upb_enumdef *enumdef = upb_filedef_enum(file, i);
+    CREATE_HASHTABLE_VALUE(desc, desc_php, EnumDescriptor, enum_descriptor_type);
+    desc->intern = SYS_MALLOC(EnumDescriptorInternal);
+    desc->intern->enumdef = enumdef;
+    desc->intern->klass = NULL;
+    desc->intern->use_nested_submsg = use_nested_submsg;
+    desc->intern->classname = NULL;
+
+    add_def_obj(desc->intern->enumdef, desc_php);
+    add_enumdef_enumdesc(desc->intern->enumdef, desc->intern);
+    fill_classname_for_desc(desc->intern, true);
+    add_class_enumdesc(desc->intern->classname, desc->intern);
+  }
+}
+
+const bool parse_and_add_descriptor(const char *data,
+                                    PHP_PROTO_SIZE data_len,
+                                    InternalDescriptorPoolImpl *pool,
+                                    upb_arena *arena,
+                                    bool use_nested_submsg TSRMLS_DC) {
+  size_t i, n;
+  google_protobuf_FileDescriptorSet *set;
+  const google_protobuf_FileDescriptorProto* const* files;
+  const upb_filedef* file;
+  upb_status status;
+
+  set = google_protobuf_FileDescriptorSet_parse(
+      data, data_len, arena);
+
+  if (!set) {
+    zend_error(E_ERROR, "Failed to parse binary descriptor\n");
+    return false;
+  }
+
+  files = google_protobuf_FileDescriptorSet_file(set, &n);
+
+  for (i = 0; i < n; i++) {
+    // Check whether file has already been added.
+    upb_strview name = google_protobuf_FileDescriptorProto_name(files[i]);
+    // TODO(teboring): Needs another look up method which takes data and length.
+    file = upb_symtab_lookupfile2(pool->symtab, name.data, name.size);
+    if (file != NULL) {
+      continue;
     }
+
+    // The PHP code generator currently special-cases descriptor.proto.  It
+    // doesn't add it as a dependency even if the proto file actually does
+    // depend on it.
+    if (depends_on_descriptor(files[i]) &&
+        upb_symtab_lookupfile(
+            pool->symtab, "google/protobuf/descriptor.proto") ==
+                NULL) {
+      if (!parse_and_add_descriptor((char *)descriptor_proto,
+                                    descriptor_proto_len, pool, arena,
+                                    use_nested_submsg TSRMLS_CC)) {
+        return false;
+      }
+    }
+
+    upb_status_clear(&status);
+    file = upb_symtab_addfile(pool->symtab, files[i], &status);
+    check_upb_status(&status, "Unable to load descriptor");
+
+    internal_add_single_generated_file(file, pool, use_nested_submsg TSRMLS_CC);
   }
 
-  upb_filedef_unref(files[0], &pool);
-  upb_gfree(files);
+  return true;
+}
+
+void internal_add_generated_file(const char *data, PHP_PROTO_SIZE data_len,
+                                 InternalDescriptorPoolImpl *pool,
+                                 bool use_nested_submsg TSRMLS_DC) {
+  int i;
+  upb_arena *arena;
+
+  arena = upb_arena_new();
+  parse_and_add_descriptor(data, data_len, pool, arena,
+                           use_nested_submsg TSRMLS_CC);
+  upb_arena_free(arena);
+  return;
 }
 
 PHP_METHOD(InternalDescriptorPool, internalAddGeneratedFile) {
   char *data = NULL;
   PHP_PROTO_SIZE data_len;
-  upb_filedef **files;
-  size_t i;
+  zend_bool use_nested_submsg = false;
 
-  if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &data, &data_len) ==
+  if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|b",
+                            &data, &data_len, &use_nested_submsg) ==
       FAILURE) {
     return;
   }
 
   InternalDescriptorPool *pool = UNBOX(InternalDescriptorPool, getThis());
-  internal_add_generated_file(data, data_len, pool TSRMLS_CC);
+  internal_add_generated_file(data, data_len, pool->intern,
+                              use_nested_submsg TSRMLS_CC);
 }
 
 PHP_METHOD(DescriptorPool, getDescriptorByClassName) {
-  DescriptorPool *public_pool = UNBOX(DescriptorPool, getThis());
-  InternalDescriptorPool *pool = public_pool->intern;
-
   char *classname = NULL;
   PHP_PROTO_SIZE classname_len;
 
@@ -1019,29 +1039,44 @@ PHP_METHOD(DescriptorPool, getDescriptorByClassName) {
     RETURN_NULL();
   }
 
-  PHP_PROTO_HASHTABLE_VALUE desc = get_ce_obj(PHP_PROTO_CE_UNREF(pce));
-  if (desc == NULL) {
-    RETURN_NULL();
+  PHP_PROTO_HASHTABLE_VALUE desc_php = get_ce_obj(PHP_PROTO_CE_UNREF(pce));
+  if (desc_php == NULL) {
+    DescriptorInternal* intern = get_ce_desc(PHP_PROTO_CE_UNREF(pce));
+    if (intern == NULL) {
+      RETURN_NULL();
+    }
+
+#if PHP_MAJOR_VERSION < 7
+    MAKE_STD_ZVAL(desc_php);
+    ZVAL_OBJ(desc_php, descriptor_type->create_object(
+                                   descriptor_type TSRMLS_CC));
+    Z_DELREF_P(desc_php);
+#else
+    desc_php =
+        descriptor_type->create_object(descriptor_type TSRMLS_CC);
+    GC_DELREF(desc_php);
+#endif
+    Descriptor* desc = UNBOX_HASHTABLE_VALUE(Descriptor, desc_php);
+    desc->intern = intern;
+    add_def_obj(intern->msgdef, desc_php);
+    add_ce_obj(PHP_PROTO_CE_UNREF(pce), desc_php);
   }
 
-  zend_class_entry* instance_ce = HASHTABLE_VALUE_CE(desc);
+  zend_class_entry* instance_ce = HASHTABLE_VALUE_CE(desc_php);
 
   if (!instanceof_function(instance_ce, descriptor_type TSRMLS_CC)) {
     RETURN_NULL();
   }
 
 #if PHP_MAJOR_VERSION < 7
-  RETURN_ZVAL(desc, 1, 0);
+  RETURN_ZVAL(desc_php, 1, 0);
 #else
-  GC_ADDREF(desc);
-  RETURN_OBJ(desc);
+  GC_ADDREF(desc_php);
+  RETURN_OBJ(desc_php);
 #endif
 }
 
 PHP_METHOD(DescriptorPool, getEnumDescriptorByClassName) {
-  DescriptorPool *public_pool = UNBOX(DescriptorPool, getThis());
-  InternalDescriptorPool *pool = public_pool->intern;
-
   char *classname = NULL;
   PHP_PROTO_SIZE classname_len;
 
@@ -1056,21 +1091,47 @@ PHP_METHOD(DescriptorPool, getEnumDescriptorByClassName) {
     RETURN_NULL();
   }
 
-  PHP_PROTO_HASHTABLE_VALUE desc = get_ce_obj(PHP_PROTO_CE_UNREF(pce));
-  if (desc == NULL) {
-    RETURN_NULL();
+  zend_class_entry* ce = PHP_PROTO_CE_UNREF(pce);
+
+  PHP_PROTO_HASHTABLE_VALUE desc_php = get_ce_obj(ce);
+  if (desc_php == NULL) {
+#if PHP_MAJOR_VERSION < 7
+    EnumDescriptorInternal* intern = get_class_enumdesc(ce->name);
+#else
+    EnumDescriptorInternal* intern = get_class_enumdesc(ZSTR_VAL(ce->name));
+#endif
+    register_class(intern, true TSRMLS_CC);
+
+    if (intern == NULL) {
+      RETURN_NULL();
+    }
+
+#if PHP_MAJOR_VERSION < 7
+    MAKE_STD_ZVAL(desc_php);
+    ZVAL_OBJ(desc_php, enum_descriptor_type->create_object(
+                                        enum_descriptor_type TSRMLS_CC));
+    Z_DELREF_P(desc_php);
+#else
+    desc_php =
+        enum_descriptor_type->create_object(enum_descriptor_type TSRMLS_CC);
+    GC_DELREF(desc_php);
+#endif
+    EnumDescriptor* desc = UNBOX_HASHTABLE_VALUE(EnumDescriptor, desc_php);
+    desc->intern = intern;
+    add_def_obj(intern->enumdef, desc_php);
+    add_ce_obj(ce, desc_php);
   }
 
-  zend_class_entry* instance_ce = HASHTABLE_VALUE_CE(desc);
+  zend_class_entry* instance_ce = HASHTABLE_VALUE_CE(desc_php);
 
   if (!instanceof_function(instance_ce, enum_descriptor_type TSRMLS_CC)) {
     RETURN_NULL();
   }
 
 #if PHP_MAJOR_VERSION < 7
-  RETURN_ZVAL(desc, 1, 0);
+  RETURN_ZVAL(desc_php, 1, 0);
 #else
-  GC_ADDREF(desc);
-  RETURN_OBJ(desc);
+  GC_ADDREF(desc_php);
+  RETURN_OBJ(desc_php);
 #endif
 }
