@@ -13,14 +13,15 @@ Example:
 This script will download pre-built protoc or protoc plugin binaries from maven
 repository and create .zip packages suitable to be included in the github
 release page. If the target is protoc, well-known type .proto files will also be
-included. Each invocation will create 7 zip packages:
+included. Each invocation will create 8 zip packages:
   dist/<TARGET>-<VERSION_NUMBER>-win32.zip
-  dist/<TARGET>-<VERSION_NUMBER>-osx-x86_32.zip
+  dist/<TARGET>-<VERSION_NUMBER>-win64.zip
   dist/<TARGET>-<VERSION_NUMBER>-osx-x86_64.zip
   dist/<TARGET>-<VERSION_NUMBER>-linux-x86_32.zip
   dist/<TARGET>-<VERSION_NUMBER>-linux-x86_64.zip
   dist/<TARGET>-<VERSION_NUMBER>-linux-aarch_64.zip
   dist/<TARGET>-<VERSION_NUMBER>-linux-ppcle_64.zip
+  dist/<TARGET>-<VERSION_NUMBER>-linux-s390x.zip
 EOF
   exit 1
 fi
@@ -31,12 +32,13 @@ VERSION_NUMBER=$2
 # <zip file name> <binary file name> pairs.
 declare -a FILE_NAMES=( \
   win32.zip windows-x86_32.exe \
-  osx-x86_32.zip osx-x86_32.exe \
+  win64.zip windows-x86_64.exe \
   osx-x86_64.zip osx-x86_64.exe \
   linux-x86_32.zip linux-x86_32.exe \
   linux-x86_64.zip linux-x86_64.exe \
   linux-aarch_64.zip linux-aarch_64.exe \
   linux-ppcle_64.zip linux-ppcle_64.exe \
+  linux-s390x.zip linux-s390x.exe \
 )
 
 # List of all well-known types to be included.
@@ -82,7 +84,7 @@ copy the contents of the 'include' directory somewhere as well, for example
 into '/usr/local/include/'.
 
 Please refer to our official github site for more installation instructions:
-  https://github.com/google/protobuf
+  https://github.com/protocolbuffers/protobuf
 EOF
 
 mkdir -p dist
@@ -96,7 +98,7 @@ for((i=0;i<${#FILE_NAMES[@]};i+=2));do
     BINARY="$TARGET"
   fi
   BINARY_NAME=${FILE_NAMES[$(($i+1))]}
-  BINARY_URL=http://repo1.maven.org/maven2/com/google/protobuf/$TARGET/${VERSION_NUMBER}/$TARGET-${VERSION_NUMBER}-${BINARY_NAME}
+  BINARY_URL=https://repo1.maven.org/maven2/com/google/protobuf/$TARGET/${VERSION_NUMBER}/$TARGET-${VERSION_NUMBER}-${BINARY_NAME}
   if ! wget ${BINARY_URL} -O ${DIR}/bin/$BINARY &> /dev/null; then
     echo "[ERROR] Failed to download ${BINARY_URL}" >&2
     echo "[ERROR] Skipped $TARGET-${VERSION_NAME}-${ZIP_NAME}" >&2
