@@ -34,7 +34,7 @@ SymbolDatabase is the MessageFactory for messages generated at compile time,
 and makes it easy to create new instances of a registered type, given only the
 type's protocol buffer symbol name.
 
-Example usage:
+Example usage::
 
   db = symbol_database.SymbolDatabase()
 
@@ -58,6 +58,7 @@ Example usage:
 """
 
 
+from google.protobuf.internal import api_implementation
 from google.protobuf import descriptor_pool
 from google.protobuf import message_factory
 
@@ -71,39 +72,62 @@ class SymbolDatabase(message_factory.MessageFactory):
     Calls to GetSymbol() and GetMessages() will return messages registered here.
 
     Args:
-      message: a message.Message, to be registered.
+      message: A :class:`google.protobuf.message.Message` subclass (or
+        instance); its descriptor will be registered.
 
     Returns:
       The provided message.
     """
 
     desc = message.DESCRIPTOR
-    self._classes[desc.full_name] = message
-    self.pool.AddDescriptor(desc)
+    self._classes[desc] = message
+    self.RegisterMessageDescriptor(desc)
     return message
 
+  def RegisterMessageDescriptor(self, message_descriptor):
+    """Registers the given message descriptor in the local database.
+
+    Args:
+      message_descriptor (Descriptor): the message descriptor to add.
+    """
+    if api_implementation.Type() == 'python':
+      # pylint: disable=protected-access
+      self.pool._AddDescriptor(message_descriptor)
+
   def RegisterEnumDescriptor(self, enum_descriptor):
     """Registers the given enum descriptor in the local database.
 
     Args:
-      enum_descriptor: a descriptor.EnumDescriptor.
+      enum_descriptor (EnumDescriptor): The enum descriptor to register.
 
     Returns:
-      The provided descriptor.
+      EnumDescriptor: The provided descriptor.
     """
-    self.pool.AddEnumDescriptor(enum_descriptor)
+    if api_implementation.Type() == 'python':
+      # pylint: disable=protected-access
+      self.pool._AddEnumDescriptor(enum_descriptor)
     return enum_descriptor
 
+  def RegisterServiceDescriptor(self, service_descriptor):
+    """Registers the given service descriptor in the local database.
+
+    Args:
+      service_descriptor (ServiceDescriptor): the service descriptor to
+        register.
+    """
+    if api_implementation.Type() == 'python':
+      # pylint: disable=protected-access
+      self.pool._AddServiceDescriptor(service_descriptor)
+
   def RegisterFileDescriptor(self, file_descriptor):
     """Registers the given file descriptor in the local database.
 
     Args:
-      file_descriptor: a descriptor.FileDescriptor.
-
-    Returns:
-      The provided descriptor.
+      file_descriptor (FileDescriptor): The file descriptor to register.
     """
-    self.pool.AddFileDescriptor(file_descriptor)
+    if api_implementation.Type() == 'python':
+      # pylint: disable=protected-access
+      self.pool._InternalAddFileDescriptor(file_descriptor)
 
   def GetSymbol(self, symbol):
     """Tries to find a symbol in the local database.
@@ -112,7 +136,7 @@ class SymbolDatabase(message_factory.MessageFactory):
     may be extended in future to support other symbol types.
 
     Args:
-      symbol: A str, a protocol buffer symbol.
+      symbol (str): a protocol buffer symbol.
 
     Returns:
       A Python class corresponding to the symbol.
@@ -121,7 +145,7 @@ class SymbolDatabase(message_factory.MessageFactory):
       KeyError: if the symbol could not be found.
     """
 
-    return self._classes[symbol]
+    return self._classes[self.pool.FindMessageTypeByName(symbol)]
 
   def GetMessages(self, files):
     # TODO(amauryfa): Fix the differences with MessageFactory.
@@ -129,10 +153,11 @@ class SymbolDatabase(message_factory.MessageFactory):
 
     Only messages already created and registered will be returned; (this is the
     case for imported _pb2 modules)
-    But unlike MessageFactory, this version also returns nested messages.
+    But unlike MessageFactory, this version also returns already defined nested
+    messages, but does not register any message extensions.
 
     Args:
-      files: The file names to extract messages from.
+      files (list[str]): The file names to extract messages from.
 
     Returns:
       A dictionary mapping proto names to the message classes.
@@ -141,20 +166,20 @@ class SymbolDatabase(message_factory.MessageFactory):
       KeyError: if a file could not be found.
     """
 
-    def _GetAllMessageNames(desc):
+    def _GetAllMessages(desc):
       """Walk a message Descriptor and recursively yields all message names."""
-      yield desc.full_name
+      yield desc
       for msg_desc in desc.nested_types:
-        for full_name in _GetAllMessageNames(msg_desc):
-          yield full_name
+        for nested_desc in _GetAllMessages(msg_desc):
+          yield nested_desc
 
     result = {}
     for file_name in files:
       file_desc = self.pool.FindFileByName(file_name)
       for msg_desc in file_desc.message_types_by_name.values():
-        for full_name in _GetAllMessageNames(msg_desc):
+        for desc in _GetAllMessages(msg_desc):
           try:
-            result[full_name] = self._classes[full_name]
+            result[desc.full_name] = self._classes[desc]
           except KeyError:
             # This descriptor has no registered class, skip it.
             pass
