@@ -161,12 +161,6 @@ void FieldGeneratorBase::GenerateExtensionCode(io::Printer* printer) {
   // and repeated fields need this default is to not generate any code
 }
 
-void FieldGeneratorBase::GenerateParsingCode(io::Printer* printer, bool use_parse_context) {
-  // for some field types the value of "use_parse_context" doesn't matter,
-  // so we fallback to the default implementation.
-  GenerateParsingCode(printer);
-}
-
 void FieldGeneratorBase::AddDeprecatedFlag(io::Printer* printer) {
   if (descriptor_->options().deprecated()) {
     printer->Print("[global::System.ObsoleteAttribute]\n");
