@@ -32,21 +32,19 @@
 //  Based on original Protocol Buffers design by
 //  Sanjay Ghemawat, Jeff Dean, and others.
 
+#include <google/protobuf/compiler/java/java_helpers.h>
+
 #include <algorithm>
 #include <limits>
 #include <unordered_set>
 #include <vector>
 
 #include <google/protobuf/stubs/stringprintf.h>
-#include <google/protobuf/compiler/java/java_helpers.h>
 #include <google/protobuf/compiler/java/java_name_resolver.h>
 #include <google/protobuf/descriptor.pb.h>
 #include <google/protobuf/wire_format.h>
 #include <google/protobuf/stubs/strutil.h>
 #include <google/protobuf/stubs/substitute.h>
-
-
-
 #include <google/protobuf/stubs/hash.h>  // for hash<T *>
 
 namespace google {
@@ -430,6 +428,7 @@ const char* BoxedPrimitiveTypeName(const FieldDescriptor* descriptor) {
   return BoxedPrimitiveTypeName(GetJavaType(descriptor));
 }
 
+
 std::string GetOneofStoredType(const FieldDescriptor* field) {
   const JavaType javaType = GetJavaType(field);
   switch (javaType) {
@@ -965,6 +964,7 @@ int GetExperimentalJavaFieldType(const FieldDescriptor* field) {
   static const int kUtf8CheckBit = 0x200;
   static const int kCheckInitialized = 0x400;
   static const int kMapWithProto2EnumValue = 0x800;
+  static const int kHasHasBit = 0x1000;
   int extra_bits = field->is_required() ? kRequiredBit : 0;
   if (field->type() == FieldDescriptor::TYPE_STRING && CheckUtf8(field)) {
     extra_bits |= kUtf8CheckBit;
@@ -973,9 +973,12 @@ int GetExperimentalJavaFieldType(const FieldDescriptor* field) {
                                HasRequiredFields(field->message_type()))) {
     extra_bits |= kCheckInitialized;
   }
+  if (HasHasbit(field)) {
+    extra_bits |= kHasHasBit;
+  }
 
   if (field->is_map()) {
-    if (SupportFieldPresence(field->file())) {
+    if (!SupportUnknownEnumValue(field)) {
       const FieldDescriptor* value =
           field->message_type()->FindFieldByName("value");
       if (GetJavaType(value) == JAVATYPE_ENUM) {
@@ -987,7 +990,7 @@ int GetExperimentalJavaFieldType(const FieldDescriptor* field) {
     return GetExperimentalJavaFieldTypeForPacked(field);
   } else if (field->is_repeated()) {
     return GetExperimentalJavaFieldTypeForRepeated(field) | extra_bits;
-  } else if (field->containing_oneof() != NULL) {
+  } else if (IsRealOneof(field)) {
     return (GetExperimentalJavaFieldTypeForSingular(field) +
             kOneofFieldTypeOffset) |
            extra_bits;
