@@ -48,7 +48,8 @@
 #include <google/protobuf/io/printer.h>
 #include <google/protobuf/io/zero_copy_stream_impl.h>
 #include <google/protobuf/io/io_win32.h>
-#include <google/protobuf/stubs/port.h>
+#include <google/protobuf/port.h>
+#include <google/protobuf/stubs/common.h>
 #include <google/protobuf/stubs/strutil.h>
 
 // NOTE: src/google/protobuf/compiler/plugin.cc makes use of cerr for some
@@ -584,6 +585,14 @@ string OneofNameCapitalized(const OneofDescriptor* descriptor) {
   return result;
 }
 
+string ObjCClass(const string& class_name) {
+  return string("GPBObjCClass(") + class_name + ")";
+}
+
+string ObjCClassDeclaration(const string& class_name) {
+  return string("GPBObjCClassDeclaration(") + class_name + ");";
+}
+
 string UnCamelCaseFieldName(const string& name, const FieldDescriptor* field) {
   string worker(name);
   if (HasSuffixString(worker, "_p")) {
@@ -909,7 +918,7 @@ bool HasNonZeroDefaultValue(const FieldDescriptor* field) {
 
 string BuildFlagsString(const FlagType flag_type,
                         const std::vector<string>& strings) {
-  if (strings.size() == 0) {
+  if (strings.empty()) {
     return GetZeroEnumNameForFlagType(flag_type);
   } else if (strings.size() == 1) {
     return strings[0];
@@ -931,12 +940,12 @@ string BuildCommentsString(const SourceLocation& location,
                                ? location.trailing_comments
                                : location.leading_comments;
   std::vector<string> lines;
-  SplitStringAllowEmpty(comments, "\n", &lines);
+  lines = Split(comments, "\n", false);
   while (!lines.empty() && lines.back().empty()) {
     lines.pop_back();
   }
   // If there are no comments, just return an empty string.
-  if (lines.size() == 0) {
+  if (lines.empty()) {
     return "";
   }
 
@@ -1292,14 +1301,14 @@ class DecodeDataBuilder {
   }
 
  private:
-  static const uint8 kAddUnderscore = 0x80;
+  static constexpr uint8 kAddUnderscore = 0x80;
 
-  static const uint8 kOpAsIs        = 0x00;
-  static const uint8 kOpFirstUpper  = 0x40;
-  static const uint8 kOpFirstLower  = 0x20;
-  static const uint8 kOpAllUpper    = 0x60;
+  static constexpr uint8 kOpAsIs = 0x00;
+  static constexpr uint8 kOpFirstUpper = 0x40;
+  static constexpr uint8 kOpFirstLower = 0x20;
+  static constexpr uint8 kOpAllUpper = 0x60;
 
-  static const int kMaxSegmentLen     = 0x1f;
+  static constexpr int kMaxSegmentLen = 0x1f;
 
   void AddChar(const char desired) {
     ++segment_len_;
@@ -1396,7 +1405,7 @@ string DirectDecodeString(const string& str) {
 // static
 string TextFormatDecodeData::DecodeDataForString(const string& input_for_decode,
                                                  const string& desired_output) {
-  if ((input_for_decode.size() == 0) || (desired_output.size() == 0)) {
+  if (input_for_decode.empty() || desired_output.empty()) {
     std::cerr << "error: got empty string for making TextFormat data, input: \""
          << input_for_decode << "\", desired: \"" << desired_output << "\"."
          << std::endl;
@@ -1506,7 +1515,7 @@ bool Parser::ParseLoop() {
     ++line_;
     RemoveComment(&line);
     TrimWhitespace(&line);
-    if (line.size() == 0) {
+    if (line.empty()) {
       continue;  // Blank line.
     }
     if (!line_consumer_->ConsumeLine(line, &error_str_)) {
@@ -1569,16 +1578,15 @@ ImportWriter::~ImportWriter() {}
 
 void ImportWriter::AddFile(const FileDescriptor* file,
                            const string& header_extension) {
-  const string file_path(FilePath(file));
-
   if (IsProtobufLibraryBundledProtoFile(file)) {
     // The imports of the WKTs are only needed within the library itself,
     // in other cases, they get skipped because the generated code already
     // import GPBProtocolBuffers.h and hence proves them.
     if (include_wkt_imports_) {
-      protobuf_framework_imports_.push_back(
-          FilePathBasename(file) + header_extension);
-      protobuf_non_framework_imports_.push_back(file_path + header_extension);
+      const string header_name =
+        "GPB" + FilePathBasename(file) + header_extension;
+      protobuf_framework_imports_.push_back(header_name);
+      protobuf_non_framework_imports_.push_back(header_name);
     }
     return;
   }
@@ -1604,7 +1612,7 @@ void ImportWriter::AddFile(const FileDescriptor* file,
     return;
   }
 
-  other_imports_.push_back(file_path + header_extension);
+  other_imports_.push_back(FilePath(file) + header_extension);
 }
 
 void ImportWriter::Print(io::Printer* printer) const {
@@ -1613,7 +1621,7 @@ void ImportWriter::Print(io::Printer* printer) const {
 
   bool add_blank_line = false;
 
-  if (protobuf_framework_imports_.size() > 0) {
+  if (!protobuf_framework_imports_.empty()) {
     const string framework_name(ProtobufLibraryFrameworkName);
     const string cpp_symbol(ProtobufFrameworkImportSymbol(framework_name));
 
@@ -1641,7 +1649,7 @@ void ImportWriter::Print(io::Printer* printer) const {
     add_blank_line = true;
   }
 
-  if (other_framework_imports_.size() > 0) {
+  if (!other_framework_imports_.empty()) {
     if (add_blank_line) {
       printer->Print("\n");
     }
@@ -1656,7 +1664,7 @@ void ImportWriter::Print(io::Printer* printer) const {
     add_blank_line = true;
   }
 
-  if (other_imports_.size() > 0) {
+  if (!other_imports_.empty()) {
     if (add_blank_line) {
       printer->Print("\n");
     }
@@ -1708,7 +1716,7 @@ bool ImportWriter::ProtoFrameworkCollector::ConsumeLine(
 
     StringPiece proto_file = proto_file_list.substr(start, offset - start);
     TrimWhitespace(&proto_file);
-    if (proto_file.size() != 0) {
+    if (!proto_file.empty()) {
       std::map<string, string>::iterator existing_entry =
           map_->find(string(proto_file));
       if (existing_entry != map_->end()) {
