@@ -125,7 +125,7 @@ bool ContainsPythonKeyword(const std::string& module_name) {
   return false;
 }
 
-inline bool IsPythonKeyword(const std::string& name) {
+inline bool IsPythonKeyword(const string& name) {
   return (std::find(kKeywords, kKeywordsEnd, name) != kKeywordsEnd);
 }
 
@@ -183,7 +183,7 @@ void PrintTopBoilerplate(io::Printer* printer, const FileDescriptor* file,
       "# -*- coding: utf-8 -*-\n"
       "# Generated by the protocol buffer compiler.  DO NOT EDIT!\n"
       "# source: $filename$\n"
-      "\"\"\"Generated protocol buffer code.\"\"\"\n",
+      "\n",
       "filename", file->name());
   if (HasTopLevelEnums(file)) {
     printer->Print(
@@ -301,7 +301,7 @@ Generator::Generator() : file_(nullptr) {}
 
 Generator::~Generator() {}
 
-uint64_t Generator::GetSupportedFeatures() const {
+uint64 Generator::GetSupportedFeatures() const {
   return CodeGenerator::Feature::FEATURE_PROTO3_OPTIONAL;
 }
 
@@ -584,7 +584,7 @@ void Generator::PrintTopLevelExtensions() const {
   for (int i = 0; i < file_->extension_count(); ++i) {
     const FieldDescriptor& extension_field = *file_->extension(i);
     std::string constant_name = extension_field.name() + "_FIELD_NUMBER";
-    ToUpper(&constant_name);
+    UpperString(&constant_name);
     printer_->Print("$constant_name$ = $number$\n", "constant_name",
                     constant_name, "number",
                     StrCat(extension_field.number()));
@@ -1262,7 +1262,7 @@ std::string Generator::ModuleLevelDescriptorName(
   // The C++ implementation doesn't guard against this either.  Leaving
   // it for now...
   std::string name = NamePrefixedWithNestedTypes(descriptor, "_");
-  ToUpper(&name);
+  UpperString(&name);
   // Module-private for now.  Easy to make public later; almost impossible
   // to make private later.
   name = "_" + name;
@@ -1292,7 +1292,7 @@ std::string Generator::ModuleLevelMessageName(
 std::string Generator::ModuleLevelServiceDescriptorName(
     const ServiceDescriptor& descriptor) const {
   std::string name = descriptor.name();
-  ToUpper(&name);
+  UpperString(&name);
   name = "_" + name;
   if (descriptor.file() != file_) {
     name = ModuleAlias(descriptor.file()->name()) + "." + name;
