@@ -33,7 +33,6 @@
 //  Sanjay Ghemawat, Jeff Dean, and others.
 
 #include <google/protobuf/stubs/casts.h>
-
 #include <google/protobuf/stubs/strutil.h>
 #include <google/protobuf/test_util.h>
 #include <google/protobuf/test_util2.h>
@@ -822,7 +821,7 @@ TEST(ExtensionSetTest, SpaceUsedExcludingSelf) {
     const int old_capacity =                                                  \
         message.GetRepeatedExtension(unittest::repeated_##type##_extension)   \
             .Capacity();                                                      \
-    EXPECT_GE(old_capacity, kMinRepeatedFieldAllocationSize);                 \
+    EXPECT_GE(old_capacity, kRepeatedFieldLowerClampLimit);                   \
     for (int i = 0; i < 16; ++i) {                                            \
       message.AddExtension(unittest::repeated_##type##_extension, value);     \
     }                                                                         \
@@ -865,7 +864,7 @@ TEST(ExtensionSetTest, SpaceUsedExcludingSelf) {
       message.AddExtension(unittest::repeated_string_extension, value);
     }
     min_expected_size +=
-        (sizeof(value) + value.size()) * (16 - kMinRepeatedFieldAllocationSize);
+        (sizeof(value) + value.size()) * (16 - kRepeatedFieldLowerClampLimit);
     EXPECT_LE(min_expected_size, message.SpaceUsed());
   }
   // Repeated messages
@@ -881,7 +880,7 @@ TEST(ExtensionSetTest, SpaceUsedExcludingSelf) {
           ->CopyFrom(prototype);
     }
     min_expected_size +=
-        (16 - kMinRepeatedFieldAllocationSize) * prototype.SpaceUsed();
+        (16 - kRepeatedFieldLowerClampLimit) * prototype.SpaceUsed();
     EXPECT_LE(min_expected_size, message.SpaceUsed());
   }
 }
@@ -1320,6 +1319,13 @@ TEST(ExtensionSetTest, DynamicExtensions) {
   }
 }
 
+TEST(ExtensionSetTest, BoolExtension) {
+  unittest::TestAllExtensions msg;
+  uint8 wire_bytes[2] = {13 * 8, 42 /* out of bounds payload for bool */};
+  EXPECT_TRUE(msg.ParseFromArray(wire_bytes, 2));
+  EXPECT_TRUE(msg.GetExtension(protobuf_unittest::optional_bool_extension));
+}
+
 }  // namespace
 }  // namespace internal
 }  // namespace protobuf
