@@ -37,9 +37,12 @@
 #include <google/protobuf/util/internal/utility.h>
 #include <google/protobuf/util/type_resolver.h>
 #include <google/protobuf/stubs/strutil.h>
-
 #include <google/protobuf/stubs/status.h>
 
+// clang-format off
+#include <google/protobuf/port_def.inc>
+// clang-format on
+
 namespace google {
 namespace protobuf {
 namespace util {
@@ -66,12 +69,12 @@ using util::error::NOT_FOUND;
 
 class DescriptorPoolTypeResolver : public TypeResolver {
  public:
-  DescriptorPoolTypeResolver(const string& url_prefix,
+  DescriptorPoolTypeResolver(const std::string& url_prefix,
                              const DescriptorPool* pool)
       : url_prefix_(url_prefix), pool_(pool) {}
 
-  Status ResolveMessageType(const string& type_url, Type* type) override {
-    string type_name;
+  Status ResolveMessageType(const std::string& type_url, Type* type) override {
+    std::string type_name;
     Status status = ParseTypeUrl(type_url, &type_name);
     if (!status.ok()) {
       return status;
@@ -86,8 +89,9 @@ class DescriptorPoolTypeResolver : public TypeResolver {
     return Status();
   }
 
-  Status ResolveEnumType(const string& type_url, Enum* enum_type) override {
-    string type_name;
+  Status ResolveEnumType(const std::string& type_url,
+                         Enum* enum_type) override {
+    std::string type_name;
     Status status = ParseTypeUrl(type_url, &type_name);
     if (!status.ok()) {
       return status;
@@ -210,7 +214,7 @@ class DescriptorPoolTypeResolver : public TypeResolver {
                 : reflection->GetBool(options, field)));
         return;
       case FieldDescriptor::CPPTYPE_STRING: {
-        const string& val =
+        const std::string& val =
             field->is_repeated()
                 ? reflection->GetRepeatedString(options, field, index)
                 : reflection->GetString(options, field);
@@ -293,15 +297,15 @@ class DescriptorPoolTypeResolver : public TypeResolver {
     ConvertEnumOptions(descriptor->options(), enum_type->mutable_options());
   }
 
-  string GetTypeUrl(const Descriptor* descriptor) {
+  std::string GetTypeUrl(const Descriptor* descriptor) {
     return url_prefix_ + "/" + descriptor->full_name();
   }
 
-  string GetTypeUrl(const EnumDescriptor* descriptor) {
+  std::string GetTypeUrl(const EnumDescriptor* descriptor) {
     return url_prefix_ + "/" + descriptor->full_name();
   }
 
-  Status ParseTypeUrl(const string& type_url, string* type_name) {
+  Status ParseTypeUrl(const string& type_url, std::string* type_name) {
     if (type_url.substr(0, url_prefix_.size() + 1) != url_prefix_ + "/") {
       return Status(
           util::error::INVALID_ARGUMENT,
@@ -312,7 +316,7 @@ class DescriptorPoolTypeResolver : public TypeResolver {
     return Status();
   }
 
-  string DefaultValueAsString(const FieldDescriptor* descriptor) {
+  std::string DefaultValueAsString(const FieldDescriptor* descriptor) {
     switch (descriptor->cpp_type()) {
       case FieldDescriptor::CPPTYPE_INT32:
         return StrCat(descriptor->default_value_int32());
@@ -352,13 +356,13 @@ class DescriptorPoolTypeResolver : public TypeResolver {
     return "";
   }
 
-  string url_prefix_;
+  std::string url_prefix_;
   const DescriptorPool* pool_;
 };
 
 }  // namespace
 
-TypeResolver* NewTypeResolverForDescriptorPool(const string& url_prefix,
+TypeResolver* NewTypeResolverForDescriptorPool(const std::string& url_prefix,
                                                const DescriptorPool* pool) {
   return new DescriptorPoolTypeResolver(url_prefix, pool);
 }
