@@ -163,13 +163,10 @@ build_golang() {
 
   export GOPATH="$HOME/gocode"
   mkdir -p "$GOPATH/src/github.com/protocolbuffers"
-  mkdir -p "$GOPATH/src/github.com/golang"
   rm -f "$GOPATH/src/github.com/protocolbuffers/protobuf"
-  rm -f "$GOPATH/src/github.com/golang/protobuf"
   ln -s "`pwd`" "$GOPATH/src/github.com/protocolbuffers/protobuf"
   export PATH="$GOPATH/bin:$PATH"
-  (cd $GOPATH/src/github.com/golang && git clone https://github.com/golang/protobuf.git && cd protobuf && git checkout v1.3.5)
-  go install github.com/golang/protobuf/protoc-gen-go
+  go get github.com/golang/protobuf/protoc-gen-go
 
   cd examples && PROTO_PATH="-I../src -I." make gotest && cd ..
 }
@@ -446,13 +443,6 @@ build_ruby27() {
 
 build_javascript() {
   internal_build_cpp
-  NODE_VERSION=node-v12.16.3-darwin-x64
-  NODE_TGZ="$NODE_VERSION.tar.gz"
-  pushd /tmp
-  curl -OL https://nodejs.org/dist/v12.16.3/$NODE_TGZ
-  tar zxvf $NODE_TGZ
-  export PATH=$PATH:`pwd`/$NODE_VERSION/bin
-  popd
   cd js && npm install && npm test && cd ..
   cd conformance && make test_nodejs && cd ..
 }
@@ -492,12 +482,24 @@ generate_php_test_proto() {
 use_php() {
   VERSION=$1
   export PATH=/usr/local/php-${VERSION}/bin:$PATH
+  export CPLUS_INCLUDE_PATH=/usr/local/php-${VERSION}/include/php/main:/usr/local/php-${VERSION}/include/php/:$CPLUS_INCLUDE_PATH
+  export C_INCLUDE_PATH=/usr/local/php-${VERSION}/include/php/main:/usr/local/php-${VERSION}/include/php/:$C_INCLUDE_PATH
   generate_php_test_proto
 }
 
 use_php_zts() {
   VERSION=$1
   export PATH=/usr/local/php-${VERSION}-zts/bin:$PATH
+  export CPLUS_INCLUDE_PATH=/usr/local/php-${VERSION}-zts/include/php/main:/usr/local/php-${VERSION}-zts/include/php/:$CPLUS_INCLUDE_PATH
+  export C_INCLUDE_PATH=/usr/local/php-${VERSION}-zts/include/php/main:/usr/local/php-${VERSION}-zts/include/php/:$C_INCLUDE_PATH
+  generate_php_test_proto
+}
+
+use_php_bc() {
+  VERSION=$1
+  export PATH=/usr/local/php-${VERSION}-bc/bin:$PATH
+  export CPLUS_INCLUDE_PATH=/usr/local/php-${VERSION}-bc/include/php/main:/usr/local/php-${VERSION}-bc/include/php/:$CPLUS_INCLUDE_PATH
+  export C_INCLUDE_PATH=/usr/local/php-${VERSION}-bc/include/php/main:/usr/local/php-${VERSION}-bc/include/php/:$C_INCLUDE_PATH
   generate_php_test_proto
 }
 
@@ -517,7 +519,9 @@ build_php5.5() {
 build_php5.5_c() {
   IS_64BIT=$1
   use_php 5.5
-  php/tests/test.sh
+  pushd php/tests
+  /bin/bash ./test.sh 5.5
+  popd
   pushd conformance
   if [ "$IS_64BIT" = "true" ]
   then
@@ -533,7 +537,9 @@ build_php5.5_mixed() {
   pushd php
   rm -rf vendor
   composer update
-  tests/compile_extension.sh
+  pushd tests
+  /bin/bash ./compile_extension.sh 5.5
+  popd
   php -dextension=./ext/google/protobuf/modules/protobuf.so ./vendor/bin/phpunit
   popd
 }
@@ -541,7 +547,7 @@ build_php5.5_mixed() {
 build_php5.5_zts_c() {
   IS_64BIT=$1
   use_php_zts 5.5
-  php/tests/test.sh
+  cd php/tests && /bin/bash ./test.sh 5.5-zts && cd ../..
   pushd conformance
   if [ "$IS_64BIT" = "true" ]
   then
@@ -567,7 +573,7 @@ build_php5.6() {
 build_php5.6_c() {
   IS_64BIT=$1
   use_php 5.6
-  php/tests/test.sh
+  cd php/tests && /bin/bash ./test.sh 5.6 && cd ../..
   pushd conformance
   if [ "$IS_64BIT" = "true" ]
   then
@@ -583,7 +589,9 @@ build_php5.6_mixed() {
   pushd php
   rm -rf vendor
   composer update
-  tests/compile_extension.sh
+  pushd tests
+  /bin/bash ./compile_extension.sh 5.6
+  popd
   php -dextension=./ext/google/protobuf/modules/protobuf.so ./vendor/bin/phpunit
   popd
 }
@@ -591,7 +599,7 @@ build_php5.6_mixed() {
 build_php5.6_zts_c() {
   IS_64BIT=$1
   use_php_zts 5.6
-  php/tests/test.sh
+  cd php/tests && /bin/bash ./test.sh 5.6-zts && cd ../..
   pushd conformance
   if [ "$IS_64BIT" = "true" ]
   then
@@ -607,7 +615,6 @@ build_php5.6_mac() {
   # Install PHP
   curl -s https://php-osx.liip.ch/install.sh | bash -s 5.6
   PHP_FOLDER=`find /usr/local -type d -name "php5-5.6*"`  # The folder name may change upon time
-  test ! -z "$PHP_FOLDER"
   export PATH="$PHP_FOLDER/bin:$PATH"
 
   # Install phpunit
@@ -621,8 +628,10 @@ build_php5.6_mac() {
   sudo mv valgrind /usr/local/bin/valgrind
 
   # Test
-  php/tests/test.sh
-  (cd conformance && make test_php_c)
+  cd php/tests && /bin/bash ./test.sh && cd ../..
+  pushd conformance
+  make test_php_c
+  popd
 }
 
 build_php7.0() {
@@ -632,13 +641,15 @@ build_php7.0() {
   composer update
   ./vendor/bin/phpunit
   popd
-  (cd conformance && make test_php)
+  pushd conformance
+  make test_php
+  popd
 }
 
 build_php7.0_c() {
   IS_64BIT=$1
   use_php 7.0
-  php/tests/test.sh
+  cd php/tests && /bin/bash ./test.sh 7.0 && cd ../..
   pushd conformance
   if [ "$IS_64BIT" = "true" ]
   then
@@ -654,7 +665,9 @@ build_php7.0_mixed() {
   pushd php
   rm -rf vendor
   composer update
-  tests/compile_extension.sh
+  pushd tests
+  /bin/bash ./compile_extension.sh 7.0
+  popd
   php -dextension=./ext/google/protobuf/modules/protobuf.so ./vendor/bin/phpunit
   popd
 }
@@ -662,7 +675,7 @@ build_php7.0_mixed() {
 build_php7.0_zts_c() {
   IS_64BIT=$1
   use_php_zts 7.0
-  php/tests/test.sh
+  cd php/tests && /bin/bash ./test.sh 7.0-zts && cd ../..
   pushd conformance
   if [ "$IS_64BIT" = "true" ]
   then
@@ -677,8 +690,7 @@ build_php7.0_mac() {
   generate_php_test_proto
   # Install PHP
   curl -s https://php-osx.liip.ch/install.sh | bash -s 7.0
-  PHP_FOLDER=`find /usr/local -type d -name "php5-7.0*"`  # The folder name may change upon time
-  test ! -z "$PHP_FOLDER"
+  PHP_FOLDER=`find /usr/local -type d -name "php7-7.0*"`  # The folder name may change upon time
   export PATH="$PHP_FOLDER/bin:$PATH"
 
   # Install phpunit
@@ -692,18 +704,17 @@ build_php7.0_mac() {
   sudo mv valgrind /usr/local/bin/valgrind
 
   # Test
-  php/tests/test.sh
-  (cd conformance && make test_php_c)
+  cd php/tests && /bin/bash ./test.sh && cd ../..
+  pushd conformance
+  make test_php_c
+  popd
 }
 
-build_php7.3_mac() {
+build_php7.4_mac() {
   generate_php_test_proto
   # Install PHP
-  # We can't test PHP 7.4 with these binaries yet:
-  #   https://github.com/liip/php-osx/issues/276
-  curl -s https://php-osx.liip.ch/install.sh | bash -s 7.3
-  PHP_FOLDER=`find /usr/local -type d -name "php5-7.3*"`  # The folder name may change upon time
-  test ! -z "$PHP_FOLDER"
+  curl -s https://php-osx.liip.ch/install.sh | bash -s 7.4
+  PHP_FOLDER=`find /usr/local -type d -name "php7-7.4*"`  # The folder name may change upon time
   export PATH="$PHP_FOLDER/bin:$PATH"
 
   # Install phpunit
@@ -717,8 +728,10 @@ build_php7.3_mac() {
   sudo mv valgrind /usr/local/bin/valgrind
 
   # Test
-  php/tests/test.sh
-  (cd conformance && make test_php_c)
+  cd php/tests && /bin/bash ./test.sh && cd ../..
+  pushd conformance
+  make test_php_c
+  popd
 }
 
 build_php_compatibility() {
@@ -728,7 +741,9 @@ build_php_compatibility() {
 
 build_php_multirequest() {
   use_php 7.4
-  php/tests/multirequest.sh
+  pushd php/tests
+  ./multirequest.sh
+  popd
 }
 
 build_php7.1() {
@@ -738,13 +753,15 @@ build_php7.1() {
   composer update
   ./vendor/bin/phpunit
   popd
-  (cd conformance && make test_php)
+  pushd conformance
+  make test_php
+  popd
 }
 
 build_php7.1_c() {
   IS_64BIT=$1
   use_php 7.1
-  php/tests/test.sh
+  cd php/tests && /bin/bash ./test.sh 7.1 && cd ../..
   pushd conformance
   if [ "$IS_64BIT" = "true" ]
   then
@@ -760,7 +777,9 @@ build_php7.1_mixed() {
   pushd php
   rm -rf vendor
   composer update
-  tests/compile_extension.sh
+  pushd tests
+  /bin/bash ./compile_extension.sh 7.1
+  popd
   php -dextension=./ext/google/protobuf/modules/protobuf.so ./vendor/bin/phpunit
   popd
 }
@@ -768,7 +787,7 @@ build_php7.1_mixed() {
 build_php7.1_zts_c() {
   IS_64BIT=$1
   use_php_zts 7.1
-  php/tests/test.sh
+  cd php/tests && /bin/bash ./test.sh 7.1-zts && cd ../..
   pushd conformance
   if [ "$IS_64BIT" = "true" ]
   then
@@ -786,13 +805,15 @@ build_php7.4() {
   composer update
   ./vendor/bin/phpunit
   popd
-  (cd conformance && make test_php)
+  pushd conformance
+  make test_php
+  popd
 }
 
 build_php7.4_c() {
   IS_64BIT=$1
   use_php 7.4
-  php/tests/test.sh
+  cd php/tests && /bin/bash ./test.sh 7.4 && cd ../..
   pushd conformance
   if [ "$IS_64BIT" = "true" ]
   then
@@ -801,7 +822,9 @@ build_php7.4_c() {
     make test_php_c_32
   fi
   popd
-  (cd php/ext/google/protobuf && phpize --clean)
+  pushd php/ext/google/protobuf
+  phpize --clean
+  popd
 }
 
 build_php7.4_mixed() {
@@ -809,16 +832,20 @@ build_php7.4_mixed() {
   pushd php
   rm -rf vendor
   composer update
-  tests/compile_extension.sh
+  pushd tests
+  /bin/bash ./compile_extension.sh 7.4
+  popd
   php -dextension=./ext/google/protobuf/modules/protobuf.so ./vendor/bin/phpunit
   popd
-  (cd php/ext/google/protobuf && phpize --clean)
+  pushd php/ext/google/protobuf
+  phpize --clean
+  popd
 }
 
 build_php7.4_zts_c() {
   IS_64BIT=$1
   use_php_zts 7.4
-  php/tests/test.sh
+  cd php/tests && /bin/bash ./test.sh 7.4-zts && cd ../..
   pushd conformance
   if [ "$IS_64BIT" = "true" ]
   then
@@ -827,7 +854,9 @@ build_php7.4_zts_c() {
     make test_php_c_32
   fi
   popd
-  (cd php/ext/google/protobuf && phpize --clean)
+  pushd php/ext/google/protobuf
+  phpize --clean
+  popd
 }
 
 build_php_all_32() {
